/**************************************************************
 *
 * arch/ppc/8220_io/1284.h
 *
 * Prototypes, etc. for the Alaska 1284 device driver
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * 2003-2004 (c) MontaVista Software, Inc. This file is licensed under
 * the terms of the GNU General Public License version 2.  This program
 * is licensed "as is" without any warranty of any kind, whether express
 * or implied.
 ***************************************************************/

#define ALPINE1284_MAJOR 120
#define ALPINE1284_DEVNAME "Alpine1284"

#define BYTE_GET(a) in_8((unsigned char *)a)
#define BYTE_PUT(a,b) out_8((unsigned char *)a,(unsigned char)b)
#define BYTE_BOR_REG(a, b) BYTE_PUT(a, BYTE_GET(a) | b)
#define BYTE_BAND_REG(a, b) BYTE_PUT(a, BYTE_GET(a) & b)
#define LW_GET(a) in_be32((unsigned int *)a)
#define LW_PUT(a,b) out_be32((unsigned int *)a,(unsigned int)b)
#define LW_BOR_REG(a, b) LW_PUT(a, LW_GET(a) | b)
#define LW_BAND_REG(a, b) LW_PUT(a, LW_GET(a) & b)

#define WRITEBYTE( _control_, _byte_)                   \
do {                                                    \
  *(unsigned char *)((_control_)->rcvPtr) = (_byte_);           \
  (_control_)->rcvPtr++;                                \
  if ((_control_)->rcvPtr >= (_control_)->rcvBufEnd) {  \
    (_control_)->rcvPtr = (_control_)->rcvBuf;          \
  }                                                     \
  (_control_)->bytesInRcvBuf++;                         \
} while (0)

#define MODESUPT_P1284EN    0x01
#define MODESUPT_BYTE       0x02
#define MODESUPT_EPP        0x04
#define MODESUPT_ECP        0x08
#define MODESUPT_RLE        0x10
#define MODESUPT_RESETSM    0x80

#define INTCTRL_INTEN       0x01
#define INTCTRL_THINTEN     0x02
#define INTCTRL_TIMINTEN    0x04
#define INTCTRL_TERMINTEN   0x08
#define INTCTRL_CMINTEN     0x10
#define INTCTRL_CONRDINTEN  0x20

#define BUFSTAT_EMPTY       0x01
#define BUFSTAT_FULL        0x02
#define BUFSTAT_TRANSDIR    0x04
#define BUFSTAT_START       0x08
#define BUFSTAT_FLUSH       0x10

#define OUTCTRL_SLCT_CTRL   0x01
#define OUTCTRL_ACKB_CTRL   0x02
#define OUTCTRL_PERR_CTRL   0x04
#define OUTCTRL_FAULTB_CTRL 0x08
#define OUTCTRL_BUSY        0x10
#define OUTCTRL_SLCTEPP     0x20
#define OUTCTRL_PERREPP     0x40

#define OUTSTAT_DATA_ALTER  0x80

#define COMPAT_MODE         0x00
#define NIBBLE_MODE         0x01
#define BYTE_MODE           0x02
#define EPP_MODE            0x04
#define ECP_MODE            0x08
#define RLE_MODE            0x10
#define IDREQ_MODE          0x20

#define FIFO_INT            0x01
#define TIMEOUT_INT         0x02
#define ABNORMAL_TERM_INT   0x04
#define CHANGE_MODE_INT     0x08
#define NORMAL_TERM_INT     0x10
#define COMMUNICATION_INT   0x20
#define ECP_DIR_CHANGE_INT  0x40

#define P1284_READ_REG_UB(a) BYTE_GET(a)
#define P1284_WRITE_REG_UB(a, b) BYTE_PUT(a, b)
#define P1284_BOR_REG_UB(a, b) BYTE_BOR_REG(a, b)
#define P1284_BAND_REG_UB(a, b) BYTE_BAND_REG(a, b)
#define P1284_READ_REG(a) LW_GET(a)
#define P1284_WRITE_REG(a, b) LW_PUT(a, b)
#define P1284_BOR_REG(a, b) LW_BOR_REG(a, b)
#define P1284_BAND_REG(a, b) LW_BAND_REG(a, b)

typedef struct timeval TimeVal;

typedef struct _s_PPControl {
	unsigned char open;	/* Has the port been opened TRUE/FALSE */
	int DMAactive;		/* flag set when DMA is active */
	unsigned int currentmode;	/* current 1284 mode */
	unsigned int direction;	/* current 1284 direction */

	/* input buffer handling */
	int bytesInRcvBuf;	/* number of bytes in the buffer */
	unsigned char *rcvPtr;	/* Where to put the next received byte */
	unsigned char *readPtr;	/* The next byte to be read is here */
	unsigned int rcvBufSize;	/* number of bytes in the receive buffer */
	unsigned char *rcvBuf;	/* Rcv buffer ptr must be cacheline aligned */
	unsigned char *rcvBufEnd;	/* points to 1 past last byte in the buffer */
#ifdef CONFIG_ALASKA_1284_DMA
	unsigned int PrevDMACount;	/* Used to see if any new data is coming in */
	/* Turn off DMA if it hasn't received any new data after this time */
	TimeVal DMATimeOut;
#endif
#ifdef CONFIG_DEVFS_FS
	devfs_handle_t de;
#endif
	unsigned int TrickleTime;	/* number of seconds since last read */

	/* output buffer handling */
	unsigned int XBufSize;	/* number of bytes in the buffer */
	unsigned int XInCount;	/* number of bytes put into the buffer */
	unsigned int XOutCount;	/* number of bytes sent to host */
	unsigned char *xbuffer;	/* Xmit buffer ptr must be cacheline aligned */
	int DidReverse;		/* Did we ever send data to the host? */
	unsigned char *id;	/* pointer to our ID */
	unsigned int idSize;	/* how many bytes in the ID */
	unsigned int idInCount;	/* how many bytes left to sent in the ID */
	unsigned int idOutCount;	/* # bytes of ID which have been sent */
	unsigned char *idbuffer;	/* ID buffer ptr must be cacheline aligned */
	unsigned int idbuffer_size;	/* Size of ID buffer */

	/* number of bytes we've been told to send to the host */
	int bytesTransmitted;

	int initialized;	/* set after initialization is complete */
	int busy;		/* one of the entrypoints is busy */
	unsigned char readTimeout;	/* Timeout for w9_read calls */
} s_PPControl;
