/*
 * arch/ppc/platforms/alaska.c
 *
 * Board setup routines for the Alaska (MPC8220i-RPC) board.
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * Based on Glacier version by Dale Farnsworth <dale.farnsworth@mvista.com>
 * with modifications for Alaska by Kent Borg and alaska_pin_mux.c
 * by Peter Barada.
 *
 * Copyright 2003 Motorola Inc.
 * Copyright 2003-2004 MontaVista, Software, Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/types.h>
#include <linux/major.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/irq.h>
#include <linux/seq_file.h>
#include <linux/ide.h>

#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/bootinfo.h>
#include <asm/machdep.h>
#include <asm/time.h>
#include <asm/pci-bridge.h>

#ifdef CONFIG_MTD
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/physmap.h>
#endif

#include <platforms/alaska.h>
#include <platforms/alaska_pci.h>
#ifdef CONFIG_BESTDMA
#include <mcdapi/MCD_dma.h>
#include <8220DmaApi.h>
#endif

BootParams OurParams;

u8 mpc8220_fec1_mac_addr[6];
u8 mpc8220_fec2_mac_addr[6];

#ifdef CONFIG_MTD_PARTITIONS
static struct mtd_partition alaska_intelFlash_partitions[] = {
	{
	  .name		= "kernel",
	  .offset	= 0x00000000,
	  .size		= 0x00280000,           /* 2.5MB           */
	  .mask_flags	= MTD_WRITEABLE,  /* force read-only */
	},
	{
	  .name		= "user",
	  .offset	= 0x00280000,
	  .size		= 0x00c60000,  /* 12.375MB */
	},
	{
	  .name		= "signature",
	  .offset	= 0x00fe0000,
	  .size		= 0x000020000,
	  .mask_flags	= MTD_WRITEABLE,  /* force read-only */
	}
};
#endif

static void
alaska_board_info(void *bdinfo, int bdinfo_size)
{
	memcpy((void *)&OurParams, bdinfo, bdinfo_size);
}

static int
alaska_show_cpuinfo(struct seq_file *m)
{
	seq_printf(m, "machine\t\t: Motorola Alaska\n");

	return 0;
}

static int
alaska_show_percpuinfo(struct seq_file *m, int i)
{
	/* Note that the PCI clock is the same as the input clock */

	seq_printf(m,
		"Input clock\t: %d.%02d MHz\n"
		"XLB clock\t: %d.%02d MHz\n"
		"Flexbus clock\t: %d.%02d MHz\n"
		"PCI clock\t: %d.%02d MHz\n"
		"Core clock\t: %d.%02d MHz\n"
		"VCO clock\t: %d.%02d MHz\n",
		OurParams.InputClock/1000000,
		(OurParams.InputClock/10000)%100,
		OurParams.XLBClock/1000000,
		(OurParams.XLBClock/10000)%100,
		OurParams.FlexClock/1000000,
		(OurParams.FlexClock/10000)%100,
		OurParams.InputClock/1000000,
		(OurParams.InputClock/10000)%100,
		OurParams.CoreClock/1000000,
		(OurParams.CoreClock/10000)%100,
		OurParams.VCOClock/1000000,
		(OurParams.VCOClock/10000)%100);

	return 0;
}

static void
alaska_set_pin_mux(void)
{
	unsigned char switches;

	switches = in_8((u8 *)CPLD_CFG1);

	printk(KERN_INFO "CPLN rev %d\n", CPLD_CFG1_REV_GET(switches));
	printk(KERN_INFO "CPLD switches 0x%02x\n",
	       switches & CPLD_CFG1_SWITCHES);

	if (switches & CPLD_CFG1_CF2) {
		printk(KERN_INFO "Set Pin Mux for FEC1\n");
		out_be32((u32 *)MPC8220_PCFG0, 0xaaaaaaab);
		out_be32((u32 *)MPC8220_PCFG1, 0x80000000);
		out_8((u8 *)CPLD_PHY, CPLD_PHY_SELECT_PHY1);
		out_be32((u32 *)MPC8220_PCFG0, 0x0);
		out_be32((u32 *)MPC8220_PCFG1, 0x0);
	} else {
		printk(KERN_INFO "Set Pin Mux for 1284\n");
		out_be32((u32 *)MPC8220_PCFG0, 0xaaaaaaab);
		out_be32((u32 *)MPC8220_PCFG1, 0x80000000);
		out_8((u8 *)CPLD_PHY, CPLD_PHY_SELECT_1284);
		out_be32((u32 *)MPC8220_PCFG0, 0x55555557);
		out_be32((u32 *)MPC8220_PCFG1, 0x80000000);
	}

	if (switches & CPLD_CFG1_CF3) {
		printk(KERN_INFO "Set Pin Mux for FEC2\n");
		out_be32((u32 *)MPC8220_PCFG2, 0xaaaaaa80);
		out_be32((u32 *)FPGA_CTRL2, 0x0ff0bc00);
		out_be32((u32 *)MPC8220_PCFG2, 0x0);
	} else {
		printk(KERN_INFO "Set Pin Mux for Video\n");
		out_be32((u32 *)MPC8220_PCFG2, 0xaaaaaa80);
		out_be32((u32 *)MPC8220_PEV_DEV_VCR, 0x0); /* PEV selected */
		out_be32((u32 *)FPGA_CTRL2, 0x0ff0bc80); /* PEV selected */
		out_be32((u32 *)MPC8220_PCFG2, 0x55555580);
	}

}

static void __init
alaska_setup_arch(void)
{
#ifdef CONFIG_BLK_DEV_INITRD
	if (initrd_start)
		ROOT_DEV = MKDEV(RAMDISK_MAJOR, 0); /* /dev/ram */
	else
#endif
#ifdef CONFIG_ROOT_NFS
		ROOT_DEV = to_kdev_t(0x00ff); /* /dev/nfs pseudo device */
#else
		ROOT_DEV = to_kdev_t(0x0802); /* /dev/sda2 */
#endif

#ifdef CONFIG_DUMMY_CONSOLE
	conswitchp = &dummy_con;
#endif
	printk(KERN_INFO "Motorola Alaska port by Motorola, Inc.\n");

	alaska_set_pin_mux();

#ifdef CONFIG_PCI
	alaska_find_bridges();
#endif
#ifdef CONFIG_MTD_PHYSMAP
	physmap_configure(FLASH_ADDR, FLASH_SIZE, 1, NULL);
	physmap_set_partitions(alaska_intelFlash_partitions,
		sizeof (alaska_intelFlash_partitions) /
		sizeof (alaska_intelFlash_partitions[0]));
#endif
}

static void __init
alaska_calibrate_decr(void)
{
	int freq, divisor;

	freq = OurParams.XLBClock;
	printk(KERN_INFO "Using XLB clock (%d.%02d MHz) to set up decrementer\n",
	       freq / 1000000, (freq / 10000) %100 );
	divisor = 4;
	tb_ticks_per_jiffy = freq / HZ / divisor;
	tb_to_us = mulhwu_scale_factor(freq / divisor, 1000000);
}

static void
alaska_restart(char *cmd)
{
	__cli();

	/* Requires updated CPLD RTL for soft-reset */
	out_8((u8 *) CPLD_CTRL, CPLD_CTRL_SFTRST);

	while (1);
}

static void
alaska_halt(void)
{
	__cli();
	while (1);
}

static void
alaska_power_off(void)
{
	alaska_halt();
}

/*
 * Create a (temporary) bat mapping for early access to I/O devices:
 * set BAT 3 to map 0xf0000000 to end of physical memory space.
 */
static __inline__ void
alaska_set_bat(void)
{
	mb();
	mtspr(DBAT3U, 0xf0001ffe);
	mtspr(DBAT3L, 0xf000002a);
	mb();
}

#ifdef CONFIG_SERIAL_TEXT_DEBUG

#if CONFIG_MPC8220_PSC_CONSOLE_PORT == 0
#define ALASKA_PSC_CONSOLE MPC8220_PSC
#else
#error "mpc8220 PSC for console not selected"
#endif

void
alaska_progress(char *s, unsigned short hex)
{
	struct mpc8220_psc *psc = (struct mpc8220_psc *)ALASKA_PSC_CONSOLE;
	char c;

	while ((c = *s++) != 0) {
		if (c == '\n') {
			while (!(in_be16(&psc->mpc8220_psc_status) &
				MPC8220_PSC_SR_TXRDY));
			out_8(&psc->mpc8220_psc_buffer_8, '\r');
		}
		while (!(in_be16(&psc->mpc8220_psc_status)
			& MPC8220_PSC_SR_TXRDY));
		out_8(&psc->mpc8220_psc_buffer_8, c);
	}
}
#endif /* CONFIG_SERIAL_TEXT_DEBUG */

/*
 * We need to read the memory controller
 * to determine this value
 *
 * Assumption: memory starts at 0x0
 */
static unsigned long __init
alaska_find_end_of_memory(void)
{
	unsigned long total = 0;
	u32 cs_cfg;

	cs_cfg = in_be32((u32 *)MPC_CS0CFG) & 0x1f;
	if (cs_cfg >= 19) {
		total = 2 << cs_cfg;
	}

	cs_cfg = in_be32((u32 *)MPC_CS1CFG) & 0x1f;
	if (cs_cfg >= 19) {
		total += 2 << cs_cfg;
	}

	cs_cfg = in_be32((u32 *)MPC_CS2CFG) & 0x1f;
	if (cs_cfg >= 19) {
		total += 2 << cs_cfg;
	}

	printk(KERN_INFO "Total memory in system: %3ld MB\n", total>>20);

	return total;
}

static void __init
alaska_map_io(void)
{
	io_block_mapping(MPC8220_MBAR, MPC8220_MBAR, 0x10000000, _PAGE_IO);
	io_block_mapping(ALASKA_PCI_SPACE_IO, ALASKA_PCI_SPACE_IO,
		(ALASKA_PCI_SPACE_IO_END - ALASKA_PCI_SPACE_IO + 1), _PAGE_IO);
}

void __init alaska_init(void)
{
	printk(KERN_INFO "Alaska Processor Version Register: %08x, System Version Register: %08x\n",
		mfspr(PVR), mfspr(SVR));
	printk(KERN_INFO "Alaska FPGA Revision: 0x%02x\n",in_be32((volatile unsigned*)FPGA_START) & 0x000000ff);
#ifdef CONFIG_BESTDMA
	initDma(); /* Loads the microcode for BestDMA */
#endif
}

void __init
platform_init(unsigned long r3, unsigned long r4, unsigned long r5,
	unsigned long r6, unsigned long r7)
{

	/* must set this before parsing boot information! */
	ppc_md.board_info = alaska_board_info;

	parse_bootinfo(find_bootinfo());

	/* ASSUMPTION:  If both r3 (BootParams pointer) and r6 (cmdline pointer)
	 * are non-zero, then we should use the board info from the BootParams
	 * structure and the cmdline pointed to by r6 instead of the
	 * information from birecs, if any.  Otherwise, use the information
	 * from birecs as discovered by the preceeding call to
	 * parse_bootinfo().  This rule should work with both U-Boot, which
	 * uses a BootParams board info structure, and the kernel boot wrapper,
	 * which uses birecs.
	 */
	if ( r3 && r6) {
		/* copy board info structure */
		alaska_board_info((void *)(r3+KERNELBASE), sizeof(BootParams));
		/* copy command line */
		*(char *)(r7+KERNELBASE) = 0;
		strcpy(cmd_line, (char *)(r6+KERNELBASE));
	}
	alaska_set_bat();

#ifdef CONFIG_BLK_DEV_INITRD
	if (r4) {
		initrd_start = r4 + KERNELBASE;
		initrd_end = r5 + KERNELBASE;
	}
#endif

	isa_io_base = ALASKA_ISA_IO_BASE;
	isa_mem_base = ALASKA_ISA_MEM_BASE;
	pci_dram_offset = ALASKA_PCI_DRAM_OFFSET;

	ppc_md.setup_arch = alaska_setup_arch;
	ppc_md.show_cpuinfo = alaska_show_cpuinfo;
	ppc_md.show_percpuinfo = alaska_show_percpuinfo;
	ppc_md.init_IRQ = alaska_init_irq;
	ppc_md.get_irq = alaska_get_irq;

	ppc_md.find_end_of_memory = alaska_find_end_of_memory;
	ppc_md.setup_io_mappings = alaska_map_io;

	ppc_md.restart = alaska_restart;
	ppc_md.power_off = alaska_power_off;
	ppc_md.halt = alaska_halt;

	/* Alaska has no timekeeper part */
	ppc_md.time_init = NULL;
	ppc_md.get_rtc_time = NULL;
	ppc_md.set_rtc_time = NULL;
	ppc_md.calibrate_decr = alaska_calibrate_decr;

#ifdef CONFIG_SERIAL_TEXT_DEBUG
	ppc_md.progress = alaska_progress;
#else
	ppc_md.progress = NULL;
#endif
	ppc_md.init = alaska_init;
}
