/*
 * CPC45 board specific definitions
 *
 * Copyright (c) 2004 Wolfgang Denk (wd@denx.de)
 */

#ifndef __PPC_PLATFORMS_CPC45_H
#define __PPC_PLATFORMS_CPC45_H

#include <asm/ppcboot.h>

/*
 * SRAM
 */
#define CFG_SRAM_PADDR  0xbfc00000
#define CFG_SRAM_SIZE   0x1fffff

/*
 * CPC45 local interrupts
 */
#define	CPC45_IRQ_LM_RTC	  0	/* LM81 / RTC	     */
#define	CPC45_IRQ_PCMCIA		1	/* PCMCIA		     */
#define	CPC45_IRQ_ETH   		2	/* Ethernet		     */
#define	CPC45_IRQ_PCI		    3	/* CPCI-IRQ(D-A)		     */

/*
 * Onboard interrupt mask registers bit map IER1
 */
#define CPC45_ONBOARD_CPCID		0x10
#define CPC45_ONBOARD_CPCIC		0x20
#define CPC45_ONBOARD_CPCIB		0x40
#define CPC45_ONBOARD_CPCIA		0x80

/*
 * Onboard interrupt mask registers bit map IER2
 */
#define CPC45_ONBOARD_SIO_1     0x0100			/* UART an X.6 */
#define CPC45_ONBOARD_SIO_2     0x0200
#define CPC45_ONBOARD_DEC       0x0400
#define CPC45_ONBOARD_LM				0x0800
#define CPC45_ONBOARD_RTC				0x1000
#define CPC45_ONBOARD_KEY_2			0x2000
#define CPC45_ONBOARD_KEY_1			0x4000
#define CPC45_ONBOARD_ABORT_AC	0x8000


#define CPC45_BCR_LED_1_ON	0x04
#define CPC45_BCR_LED_2_ON	0x08
#define CPC45_BCR_WDT_RS  	0x80	/* WDT Reset Status */

#define CPC45_BCSR_WDT_ON  	0x80	/* WDT On */

typedef volatile unsigned char	vuchar;

/*
 * CPC45 Board Control and Status registers
 */
typedef struct cpc45_bcsr_s {
	vuchar	lie1;	/* Local Interrupt Enable Register 1 */
	vuchar	lis1;	/* Local Interrupt Status Register 1 */
	vuchar	lie2;	/* Local Interrupt Enable Register 2 */
	vuchar	lis2;	/* Local Interrupt Status Register 2 */
	vuchar	bcr;	/* Board Control Register */
	vuchar	bcsr;	/* Board Control Status Register */
	vuchar	wdr;    /* Watchdog Retriggered Port */
	vuchar  dummy;
	vuchar	brr1;	/* Board Revision Register 1 */
	vuchar	brr2;	/* Board Revision Register 2 */
	vuchar  dummy2[54];
	vuchar  ddr[32];	/* Display Data Register */
} cpc45_bcsr_t;

extern volatile cpc45_bcsr_t	*bcsr;

void cpc45_find_bridges(void);

#endif /* __PPC_PLATFORMS_CPC45_H */
