/*
 * (C) Copyright 2005
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/pci.h>
#include <linux/pci_ids.h>
#include <linux/slab.h>

#include <asm/byteorder.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>
#include <asm/machdep.h>
#include <asm/pci-bridge.h>
#include <asm/mpc10x.h>

#include "cpc45.h"

extern 	void cpc45_InitDisplay (void);

volatile cpc45_bcsr_t	*bcsr;

void cpc45_pcibios_fixup(void)
{
  struct pci_dev *dev;

	pci_for_each_dev(dev)
	{
		if (dev->devfn == 0x78)
		{
			/* ETH */
			dev->irq = 2;
			pcibios_write_config_byte(dev->bus->number, dev->devfn,
			                          PCI_INTERRUPT_LINE, dev->irq);
		}
		if (dev->devfn == 0x70)
		{
			/* PCMCIA */
			dev->irq = 1;
			pcibios_write_config_byte(dev->bus->number, dev->devfn,
			                          PCI_INTERRUPT_LINE, dev->irq);
		}
	}
}

static void __init PLX9030_Init(struct pci_dev *dev, uint32_t	membaseCsr)
{
	uint32_t	membase;

	pci_read_config_dword(dev, PCI_BASE_ADDRESS_5, &membase);
	bcsr = (cpc45_bcsr_t *)ioremap(membase, 0x1000);
#ifdef DEBUG
	printk("BCSR: %p\n", bcsr);
	printk("lie1: %x\n", bcsr->lie1);
	printk("lis1: %x\n", bcsr->lis1);
	printk("lie2: %x\n", bcsr->lie2);
	printk("lis2: %x\n", bcsr->lis2);
	printk("bcr : %x\n", bcsr->bcr);
	printk("bcsr: %x\n", bcsr->bcsr);
	printk("wdr : %x\n", bcsr->wdr);
	printk("brr1: %x\n", bcsr->brr1);
	printk("brr2: %x\n", bcsr->brr2);
#endif

	if((bcsr->bcr & 0x30) != 0x30){
		/* Flash Write Enable */
		printk("Enable Flash Write ");
		bcsr->bcr |= 0x30;
		bcsr->bcr &= ~0x03;
		writeb(0x30, &bcsr->bcr);
#ifdef DEBUG
		printk("bcr : %x\n", bcsr->bcr);
#endif
	}
	/* enable SIO 1 Interrupt */
	bcsr->lie2 |= 0x01;
	cpc45_InitDisplay ();
}

static inline void cpc45_pcibios_fixup_resources(struct pci_dev *dev)
{
  uint32_t	membaseCsr;	  /* base address of device memory space */

	switch(dev->vendor){
		case PCI_VENDOR_ID_PLX:
		if(dev->device == PCI_DEVICE_ID_PLX_9030){
#ifdef DEBUG
			printk("PLX9030 device found !! %s\n", dev->name);
#endif
			pci_read_config_dword(dev, PCI_BASE_ADDRESS_0, &membaseCsr);
			PLX9030_Init(dev, membaseCsr);
		}
		break;
		case PCI_VENDOR_ID_CIRRUS:
		if (dev->device == PCI_DEVICE_ID_CIRRUS_6729){
#ifdef DEBUG
			printk("Cirrus PD6729 device found !! %s\n", dev->name);
#endif
			/* enable mapped memory and IO addresses */
			pci_write_config_dword(dev,
						   PCI_COMMAND,
						   PCI_COMMAND_MEMORY |
						   PCI_COMMAND_IO |
								 PCI_COMMAND_WAIT);
			pci_write_config_dword(dev, PCI_BASE_ADDRESS_0, 0x3e0);
		}
		break;

	}
}

void __init
cpc45_find_bridges(void)
{
	struct pci_controller	*hose;

	hose = pcibios_alloc_controller();

	if (!hose)
		return;

	hose->first_busno = 0;
	hose->last_busno = 0xff;

	if (mpc10x_bridge_init(hose,
			       MPC10X_MEM_MAP_B,
			       MPC10X_MEM_MAP_B,
			       MPC10X_MAPB_EUMB_BASE) == 0) {

		/* Scan PCI bus */
		hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);

		ppc_md.pcibios_fixup = cpc45_pcibios_fixup;
		ppc_md.pcibios_fixup_bus = NULL;
		ppc_md.pcibios_fixup_resources = cpc45_pcibios_fixup_resources;
	}
	else {
		if (ppc_md.progress)
			ppc_md.progress("Bridge init failed", 0x100);
		printk("Host bridge init failed\n");
	}

	return;
}
