/*
 * arch/ppc/platforms/mv_setup.c
 * 
 * Board setup routines for the MATRIX Vision mvBOX.
 *
 * MATRIX Vision GmbH
 * Talstr. 16
 * Oppenweiler
 * Germany
 *
 * Copyright 2002 MATRIX Vision GmbH.
 *
 * Based on Musenki routines from...
 * Author: Jim Thompson/Paul Jimenez
 *         jim@musenki.com/pj@musenki.com
 *
 * Copyright 2002 Musenki, Inc.
 *
 *
 * Based on Sandpoint routines from...
 * Author: Mark A. Greer
 *         mgreer@mvista.com
 *
 * Copyright 2000-2002 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/major.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/irq.h>
#include <linux/ide.h>
#include <linux/irq.h>
#include <linux/seq_file.h>

#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/time.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/prom.h>
#include <asm/smp.h>
#include <asm/keyboard.h>
#include <asm/vga.h>
#include <asm/open_pic.h>
#include <asm/i8259.h>
#include <asm/todc.h>
#include <asm/bootinfo.h>
#include <asm/mpc10x.h>
#include <asm/pci-bridge.h>

#include "mv.h"

#undef DUMP_DBATS

#define MB	(1024*1024)

extern int pckbd_setkeycode(unsigned int scancode, unsigned int keycode);
extern int pckbd_getkeycode(unsigned int scancode);
extern int pckbd_translate(unsigned char scancode, unsigned char *keycode,
		                           char raw_mode);
extern char pckbd_unexpected_up(unsigned char keycode);
extern void pckbd_leds(unsigned char leds);
extern void pckbd_init_hw(void);
extern unsigned char pckbd_sysrq_xlate[128];

static void	mv_halt(void);

unsigned char __res[sizeof(bd_t)];
extern char cmd_line[];

static char *mvBoardName[2] = { "mvBlueBOX\0", "mvBlueLYNX\0" };
static int BoardType = 0;
static int BusSpeed  = 0;
/* -------------------------------------------------------------------- */
/*
 * Define all of the IRQ senses and polarities.
 */
static u_char mv_openpic_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* EPIC IRQ 0 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* EPIC IRQ 1 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* EPIC IRQ 2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* EPIC IRQ 3 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* EPIC IRQ 4 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* EPIC IRQ 5 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* EPIC IRQ 6 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* EPIC IRQ 7 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* EPIC IRQ 8 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* EPIC IRQ 9 */
	(IRQ_SENSE_EDGE  | IRQ_POLARITY_NEGATIVE),  /* 10 */
	(IRQ_SENSE_EDGE  | IRQ_POLARITY_NEGATIVE),
	(IRQ_SENSE_EDGE  | IRQ_POLARITY_NEGATIVE),
	(IRQ_SENSE_EDGE  | IRQ_POLARITY_NEGATIVE) 
};

#include <linux/serial.h>
#include <linux/serialP.h>
#include <linux/serial_reg.h>
#include <asm/serial.h>

extern struct serial_state rs_table[RS_TABLE_SIZE];

#define PCI_CONFIG(b,d,f,r)		cpu_to_le32( 0x80000000 | ((b&0xff)<<16) \
														| ((d&0x1f)<<11) \
														| ((f&0x7)<<7)   \
														| (r&0xfc) )

int mv_pci_read( int bus, int dev, int func, int reg )
{
	*(u32*)(0xfec00cf8) = PCI_CONFIG(bus,dev,func,reg);
	asm("sync");
	return cpu_to_le32( *(u32*)(0xfee00cfc) );
}
/* -------------------------------------------------------------------- */
static int __init 
get_BoardType(void) 
{
    return ( mv_pci_read(0,0xe,0,0) == 0x54021131 ? 0 : 1 );
}

/* -------------------------------------------------------------------- */
static int __init
mv_get_bus_speed(void)
{
	bd_t	*binfo = (bd_t *)__res;
	return (binfo->bi_busfreq * 1000000);
}

/* -------------------------------------------------------------------- */
static void __init
mv_set_BoardDefs(void)
{
	if ( BoardType == 0) {
		mv_openpic_initsenses[0] = IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE;

		rs_table[0].baud_base = BASE_BAUD;
		rs_table[0].port = MV_SB1;
		rs_table[0].irq = SERIAL_SB1_IRQ;
		rs_table[0].iomem_base = (u8 *)MV_SB1;
		
		rs_table[1].baud_base = BASE_BAUD;
		rs_table[1].port = MV_SB0;
		rs_table[1].irq = SERIAL_SB0_IRQ;
		rs_table[1].iomem_base = (u8 *)MV_SB0;
	} else {
		rs_table[0].baud_base = BusSpeed >> 4;
		rs_table[1].baud_base = BusSpeed >> 4;
	}
}
/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static void __init
mv_setup_arch(void)
{
	loops_per_jiffy = 100000000 / HZ;

#ifdef CONFIG_BLK_DEV_INITRD
	if (initrd_start)
		ROOT_DEV = MKDEV(RAMDISK_MAJOR, 0);
	else
#endif
#ifdef	CONFIG_ROOT_NFS
		ROOT_DEV = to_kdev_t(0x00FF);	/* /dev/nfs pseudo device */
#else
		ROOT_DEV = to_kdev_t(0x0301);	/* /dev/hda1 IDE disk */
#endif

	mv_find_bridges();

	/* make sure DUART is in dual serial port mode without handshaking */
	*(((volatile unsigned char *) MV_EPIC0) + 0x11) = 1;

#ifdef CONFIG_DUMMY_CONSOLE
	conswitchp = &dummy_con;
#endif

	printk(KERN_INFO "MATRIX VISION %s\n", mvBoardName[BoardType] );
	printk(KERN_INFO "Port by MATRIX VISION GmbH. (info@matrix-vision.de)\n");


#ifdef CONFIG_SERIAL_TEXT_DEBUG
	if (ppc_md.progress)
		ppc_md.progress("mv_setup_arch: exit", 0);
#endif
}

/* -------------------------------------------------------------------- */
static void __init
mv_init2(void)
{
	request_region(0x00,0x20,"dma1");
	request_region(0x20,0x20,"pic1");
	request_region(0x40,0x20,"timer");
	request_region(0x80,0x10,"dma page reg");
	request_region(0xa0,0x20,"pic2");
	request_region(0xc0,0x20,"dma2");

	return;
}

/* -------------------------------------------------------------------- */
/*
 * Interrupt setup and service.  Interrrupts on the mvBOX come
 * from the Davicom chip, Trimedia and PCI slot _or_ the 8259 in the VIA bridge.
 * The 8259 is cascaded from EPIC IRQ0, IRQ1 maps to Davicom
 * IRQ2 maps to Trimedia, IRQ3 maps to the PCI slot.
 */
static void __init
mv_init_IRQ(void)
{
	int i;

	OpenPIC_InitSenses = mv_openpic_initsenses;
	OpenPIC_NumInitSenses = sizeof(mv_openpic_initsenses);

	/* Map EPIC IRQs 0-4 */
	openpic_set_sources(0, 4, OpenPIC_Addr + 0x10200);
	/* Skip reserved space and map i2c and DMA Ch[01] */
	openpic_set_sources(4, 3, OpenPIC_Addr + 0x11020);
	/* Skip reserved space and map Message Unit Interrupt (I2O) */
	openpic_set_sources(7, 1, OpenPIC_Addr + 0x110C0);
	/* Skip reserved space and map DUART Interrupts */
	openpic_set_sources(8, 2, OpenPIC_Addr + 0x11120);

	/* map timer interrupts */
	openpic_set_sources( 10, 1, OpenPIC_Addr + 0x01120);
	openpic_set_sources( 11, 1, OpenPIC_Addr + 0x01160);
	openpic_set_sources( 12, 1, OpenPIC_Addr + 0x011a0);
	openpic_set_sources( 13, 1, OpenPIC_Addr + 0x011e0);

	//set timer frequency reporting register (TFRR) 
	*(int *)( OpenPIC_Addr + 0x10f0 ) = cpu_to_le32( BusSpeed >> 3 );

	switch ( BoardType ) {
		case 0: /* mvBLUEBox */
			openpic_init (NUM_8259_INTERRUPTS);
			openpic_hookup_cascade (NUM_8259_INTERRUPTS, "8259 cascade to EPIC", &i8259_irq);
			for(i=0; i < NUM_8259_INTERRUPTS; i++) 
				irq_desc[i].handler = &i8259_pic;
			i8259_init(0xfef00000);
			break;
		case 1: /* mvBlueLYNX */
			openpic_init(0);
			break;
		default:
			printk("* ERROR: no BoardType defined !\n");
			return;
	}
}

static int
mvBox_get_irq(struct pt_regs *regs)
{
    int irq, cascade_irq;

    irq = openpic_irq();

    if (irq == NUM_8259_INTERRUPTS) {
        cascade_irq = i8259_irq(regs);

        if (cascade_irq != -1) {
            irq = cascade_irq;
            openpic_eoi();
        }
    } else if (irq == OPENPIC_VEC_SPURIOUS + 16)
        irq = -1;

    return irq;
}

static int
mvSensor2_get_irq(struct pt_regs *regs)
{
	int irq = openpic_irq();
	if (irq == OPENPIC_VEC_SPURIOUS )
		return -1;
	return irq;
}

#ifdef DUMP_DBATS
static void print_dbat(int idx, u32 bat) {

	char str[64];

	sprintf(str, "DBAT%c%c = 0x%08x",
		(char)((idx - DBAT0U) / 2) + '0',
		(idx & 1) ? 'L' : 'U', bat);
	ppc_md.progress(str, 0);
}

#define DUMP_DBAT(x) \
	do { \
	u32 __temp = mfspr(x);\
	print_dbat(x, __temp); \
	} while (0)

/* -------------------------------------------------------------------- */
static void dump_dbats(void) {

	if (ppc_md.progress) {
		DUMP_DBAT(DBAT0U);
		DUMP_DBAT(DBAT0L);
		DUMP_DBAT(DBAT1U);
		DUMP_DBAT(DBAT1L);
		DUMP_DBAT(DBAT2U);
		DUMP_DBAT(DBAT2L);
		DUMP_DBAT(DBAT3U);
		DUMP_DBAT(DBAT3L);
	}
}
#endif


/* -------------------------------------------------------------------- */
#define DEFAULT_SUBTRACT (4 * MB)
static unsigned long parse_allocate_option (char *line)
{
	/* Look for allocate= option on command line */
	if (strstr(cmd_line, "allocate="))
	{
		char *p, *q;
		unsigned long subtract = DEFAULT_SUBTRACT;

		for (q = cmd_line; (p = strstr(q, "allocate=")) != 0; )
		{
			q = p + 9;	/* strlen ("allocate=") */
			if (p > cmd_line && p[-1] != ' ')
				continue;
			subtract = simple_strtoul(q, &q, 0);
			if (*q == 'k' || *q == 'K')
			{
				subtract <<= 10;
				++q;
			} else if (*q == 'm' || *q == 'M')
			{
				subtract <<= 20;
				++q;
			}
		}
		return subtract;
	}
	return DEFAULT_SUBTRACT;
}


/* -------------------------------------------------------------------- */
static unsigned long __init
mv_find_end_of_memory(void)
{
	bd_t	*binfo;
	extern unsigned char __res[];
	
	binfo = (bd_t *)__res;

   	return (binfo->bi_memsize - (parse_allocate_option (cmd_line)));
}


/* -------------------------------------------------------------------- */
static void __init
mv_map_io(void)
{
	/*map 0xf0000000 to end: 1-to-1, cache inhibit, guarded */
	io_block_mapping(0xf0000000, 0xf0000000, 0x10000000, _PAGE_IO);
#ifdef DUMP_DBATS
	if (ppc_md.progress)
		ppc_md.progress("mv_map_io: exit", 0);
    dump_dbats();
#endif

}

/* -------------------------------------------------------------------- */
/* The decrementer counts at the system (internal) clock frequency
 * divided by four.
 */
void __init mv_calibrate_decr(void)
{
	int divisor = 4; /* Decrementer = time base = sys_logic_clk/4 */

	printk(KERN_INFO "freq = %d\n", BusSpeed );
	tb_ticks_per_jiffy = BusSpeed / HZ / divisor;
	tb_to_us = mulhwu_scale_factor(BusSpeed / divisor, 1000000);
}

/* -------------------------------------------------------------------- */
static void
mv_restart(char *cmd)
{
	//trigger mv watchdog
	*(volatile unsigned char *) (0xff000005) = 0x1;
}

/* -------------------------------------------------------------------- */
static void
mv_power_off(void)
{
	printk("CPU halted !\n");
	for(;;){}
}

/* -------------------------------------------------------------------- */
static void
mv_halt(void)
{
   mv_restart(NULL);
}

/* -------------------------------------------------------------------- */
static int
mv_show_cpuinfo(struct seq_file *m)
{
	uint pvid = mfspr(PVR);

    seq_printf(m, "vendor\t\t: MATRIX VISION GmbH\n");
    seq_printf(m, "machine\t\t: %s\n", mvBoardName[BoardType] );
    seq_printf(m, "cpu made by\t: %s\n", (pvid & (1<<15) ? "IBM" : "Motorola"));
    seq_printf(m, "bus speed\t: %d MHz\n", BusSpeed/1000000);
	return 0;
}

/* -------------------------------------------------------------------- */
/*
 * Set BAT 3 to map 0xf0000000 to end of physical memory space 1-to-1.
 */
static __inline__ void
mv_set_bat(void)
{
	unsigned long bat3u, bat3l;

	__asm__ __volatile__(
			" lis %0,0xf000\n	\
			ori %1,%0,0x002a\n	\
			ori %0,%0,0x1fff\n	\
			mtspr 0x21e,%0\n	\
			mtspr 0x21f,%1\n	\
			isync\n			\
			sync "
			: "=r" (bat3u), "=r" (bat3l));
}


#ifdef CONFIG_SERIAL_TEXT_DEBUG
volatile unsigned char *com_port;
volatile unsigned char *com_port_lsr;

/* -------------------------------------------------------------------- */
static void
serial_writechar(char c)
{
	while ((*com_port_lsr & UART_LSR_THRE) == 0)
		;
	*com_port = c;
}

/* -------------------------------------------------------------------- */
void
mv_progress(char *s, unsigned short hex)
{
	volatile char c;

	com_port = (volatile unsigned char *) MV_EPIC0;
	com_port_lsr = com_port + UART_LSR;

	while ((c = *s++) != 0)
		serial_writechar(c);

	/* Most messages don't have a newline in them */
	serial_writechar('\n');
	serial_writechar('\r');
}
#endif	/* CONFIG_SERIAL_TEXT_DEBUG */


/* -------------------------------------------------------------------- */
TODC_ALLOC();

/* -------------------------------------------------------------------- */
void __init
platform_init(unsigned long r3, unsigned long r4, unsigned long r5,
	      unsigned long r6, unsigned long r7)
{
	bd_t	*binfo = (bd_t *)__res;
	char	console_string[64];

	/* use information passed by boot loader */
	if ( r3 )
	{	/* get boot loader information (PPCBoot) */
        memcpy( (void *)__res,(void *)(r3+KERNELBASE), sizeof(bd_t) );
	}

    /* take care of cmd line */
    if ( r6 && r7 )
    {	/* use cmd_line from boot loader (PPCBoot) */
        *(char *)(r7+KERNELBASE) = 0;
        strcpy(cmd_line, (char *)(r6+KERNELBASE));

		if (!strstr(cmd_line, "console="))
		{	/* console option not set on the command line - try to determine it from PPCBoot variables.
	   		 * N.B. this won't work	if using a standard PPCBoot header structure - you must set a 
			 * kernel option in this case! 
			 */
			sprintf (console_string, " console=ttyS0,%ldN8", binfo->bi_baudrate);
			strcat (cmd_line, console_string);
		}
	}

	/* Map in board regs, etc. */
	mv_set_bat();

	BoardType = get_BoardType();
	BusSpeed  = mv_get_bus_speed();
	mv_set_BoardDefs();

	isa_io_base = MPC10X_MAPB_ISA_IO_BASE;
	isa_mem_base = MPC10X_MAPB_ISA_MEM_BASE;
	pci_dram_offset = MPC10X_MAPB_DRAM_OFFSET;
	ISA_DMA_THRESHOLD = 0x00ffffff;
	DMA_MODE_READ = 0x44;
	DMA_MODE_WRITE = 0x48;

	ppc_md.setup_arch = mv_setup_arch;
	ppc_md.show_cpuinfo = mv_show_cpuinfo;
	ppc_md.irq_cannonicalize = NULL;
	ppc_md.init_IRQ = mv_init_IRQ;

	if ( BoardType == 0 )
		ppc_md.get_irq = mvBox_get_irq;
	else
		ppc_md.get_irq = mvSensor2_get_irq;
		
	ppc_md.init = mv_init2;

	ppc_md.restart = mv_restart;
	ppc_md.power_off = mv_power_off;
	ppc_md.halt = mv_halt;

	ppc_md.find_end_of_memory = mv_find_end_of_memory;
	ppc_md.setup_io_mappings = mv_map_io;

	ppc_md.calibrate_decr = mv_calibrate_decr;
	ppc_md.heartbeat = NULL;

#ifdef	CONFIG_SERIAL_TEXT_DEBUG
	ppc_md.progress = mv_progress;
#endif

#ifdef CONFIG_VT
	ppc_md.kbd_setkeycode    = pckbd_setkeycode;
	ppc_md.kbd_getkeycode    = pckbd_getkeycode;
	ppc_md.kbd_translate     = pckbd_translate;
	ppc_md.kbd_unexpected_up = pckbd_unexpected_up;
	ppc_md.kbd_leds          = pckbd_leds;
	ppc_md.kbd_init_hw       = pckbd_init_hw;
#ifdef CONFIG_MAGIC_SYSRQ
	ppc_md.ppc_kbd_sysrq_xlate = pckbd_sysrq_xlate;
	SYSRQ_KEY = 0x54;
#endif

#endif

	return;
}
