/*
 *
 * Authors: Kent Borg, Dale Farnsworth <dale.farnsworth@mvista.com>,
 * 	and Wolfgang Denk <wd@denx.de> and Reinhard Meyer
 *
 * Modification of icecube.c by R.Meyer
 *
 * Copyright 2003 Motorola Inc.
 * Copyright 2003 MontaVista Software Inc.
 * Copyright 2003-2004 DENX Software Engineering (wd@denx.de)
 * Copyright 2004 EMK Elektronik GmbH
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/types.h>
#include <linux/major.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/irq.h>
#include <linux/seq_file.h>
#include <linux/ide.h>

#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/bootinfo.h>
#include <asm/machdep.h>
#include <asm/mk48t59.h>
#include <asm/time.h>
#include <asm/pci-bridge.h>
#include <asm/mpc5xxx.h>
#ifdef CONFIG_UBOOT
#include <asm/ppcboot.h>
unsigned char __res[sizeof(bd_t)];
#endif

extern int mpc5xxx_setkeycode(unsigned int scancode, unsigned int keycode);
extern int mpc5xxx_getkeycode(unsigned int scancode);
extern int mpc5xxx_translate(unsigned char scancode, unsigned char *keycode,
			   char raw_mode);
extern char mpc5xxx_unexpected_up(unsigned char keycode);
extern void mpc5xxx_kbd_init_hw(void);
extern void mpc5xxx_bestcomm_init(void);

extern unsigned long mpc5xxx_find_end_of_memory(void);
extern void mpc5xxx_find_bridges(void);
extern void mpc5xxx_set_bat(void);
extern void mpc5xxx_map_io(void);
extern void mpc5xxx_restart(char *cmd);
extern void mpc5xxx_halt(void);
extern void mpc5xxx_power_off(void);
extern void mpc5xxx_ide_init_hwif_ports(hw_regs_t *hw,
		      ide_ioreg_t data_port, ide_ioreg_t ctrl_oort, int *irq);
extern ide_ioreg_t mpc5xxx_ide_get_base(int index);

#if defined (CONFIG_EVAL5200)
/* EVAL5200 has an M48T08 Timekeeper/NVRAM */

static unsigned char
local_nvram_read_val(int addr)
{
	unsigned char retval;
	
	retval = in_8((volatile unsigned char *)(0xf0010000+addr));
	return retval;
}

static void
local_nvram_write_val(int addr, unsigned char val)
{
   	out_8((volatile unsigned char *)(0xf0010000+addr), val);
}

static int
local_set_rtc_time(unsigned long nowtime)
{
	unsigned char save_control;
	struct rtc_time tm;

	spin_lock(&rtc_lock);
	to_tm(nowtime, &tm);

	/* tell the clock it's being written */
	save_control = ppc_md.nvram_read_val(MK48T59_RTC_CONTROLA);

	ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA,
			     (save_control | MK48T59_RTC_CA_WRITE));

        tm.tm_year = (tm.tm_year - 1900) % 100;
	BIN_TO_BCD(tm.tm_sec);
	BIN_TO_BCD(tm.tm_min);
	BIN_TO_BCD(tm.tm_hour);
	BIN_TO_BCD(tm.tm_mon);
	BIN_TO_BCD(tm.tm_mday);
	BIN_TO_BCD(tm.tm_year);

	ppc_md.nvram_write_val(MK48T59_RTC_SECONDS,      tm.tm_sec);
	ppc_md.nvram_write_val(MK48T59_RTC_MINUTES,      tm.tm_min);
	ppc_md.nvram_write_val(MK48T59_RTC_HOURS,        tm.tm_hour);
	ppc_md.nvram_write_val(MK48T59_RTC_MONTH,        tm.tm_mon);
	ppc_md.nvram_write_val(MK48T59_RTC_DAY_OF_MONTH, tm.tm_mday);
	ppc_md.nvram_write_val(MK48T59_RTC_YEAR,         tm.tm_year);

	/* Turn off the write bit. */
	ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA, save_control);
	spin_unlock(&rtc_lock);

	return 0;
}

static unsigned long
local_get_rtc_time(void)
{
	unsigned char save_control;
	unsigned int year, mon, day, hour, min, sec;
	unsigned long result;
	
	/* Simple: freeze the clock, read it and allow updates again */
	save_control = ppc_md.nvram_read_val(MK48T59_RTC_CONTROLA);
	save_control &= ~MK48T59_RTC_CA_READ;

	ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA, save_control);

	/* Set the register to read the value. */
	ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA,
			     (save_control | MK48T59_RTC_CA_READ));

	sec = ppc_md.nvram_read_val(MK48T59_RTC_SECONDS);
	min = ppc_md.nvram_read_val(MK48T59_RTC_MINUTES);
	hour = ppc_md.nvram_read_val(MK48T59_RTC_HOURS);
	day = ppc_md.nvram_read_val(MK48T59_RTC_DAY_OF_MONTH);
	mon = ppc_md.nvram_read_val(MK48T59_RTC_MONTH);
	year = ppc_md.nvram_read_val(MK48T59_RTC_YEAR);

	/* Let the time values change again. */
	ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA, save_control);

	BCD_TO_BIN(sec);
	BCD_TO_BIN(min);
	BCD_TO_BIN(hour);
	BCD_TO_BIN(day);
	BCD_TO_BIN(mon);
	BCD_TO_BIN(year);

	year = year + 1900;
	if (year < 1970) {
		year += 100;
	}

	result = mktime(year, mon, day, hour, min, sec);

	return result;
}

static long __init
local_init(void) {
	unsigned char tmp;

	tmp = ppc_md.nvram_read_val(MK48T59_RTC_CONTROLB);
	if (tmp & MK48T59_RTC_CB_STOP) {
		printk("Warning: RTC was stopped, date will be wrong.\n");
		ppc_md.nvram_write_val(MK48T59_RTC_CONTROLB,
					 tmp & ~MK48T59_RTC_CB_STOP);
		/* Low frequency crystal oscillators may take a very long
		 * time to startup and stabilize. For now just ignore the
		 * the issue, but attempting to calibrate the decrementer
		 * from the RTC just after this wakeup is likely to be very
		 * inaccurate. Firmware should not allow to load
		 * the OS with the clock stopped anyway...
		 */
	}
	/* Ensure that the clock registers are updated */
	tmp = ppc_md.nvram_read_val(MK48T59_RTC_CONTROLA);
	tmp &= ~(MK48T59_RTC_CA_READ | MK48T59_RTC_CA_WRITE);
	ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA, tmp);
	return 0;
}

/* We use the NVRAM RTC to time a second to calibrate the decrementer,
 * the RTC registers have just been set up in the right state by the
 * preceding routine.
 */
static void __init
local_calibrate_decr(void)
{
	unsigned long freq;
	unsigned long t1;
	unsigned char save_control;
	long i;
	unsigned char sec;


	/* Make sure the time is not stopped. */
	save_control = ppc_md.nvram_read_val(MK48T59_RTC_CONTROLB);

	ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA,
			(save_control & (~MK48T59_RTC_CB_STOP)));

	/* Now make sure the read bit is off so the value will change. */
	save_control = ppc_md.nvram_read_val(MK48T59_RTC_CONTROLA);
	save_control &= ~MK48T59_RTC_CA_READ;
	ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA, save_control);


	/* Read the seconds value to see when it changes. */
	sec = ppc_md.nvram_read_val(MK48T59_RTC_SECONDS);
	/* Actually this is bad for precision, we should have a loop in
	 * which we only read the seconds counter. nvram_read_val writes
	 * the address bytes on every call and this takes a lot of time.
	 * Perhaps an nvram_wait_change method returning a time
	 * stamp with a loop count as parameter would be the  solution.
	 */
	for (i = 0 ; i < 10000000 ; i++)	{ /* may take up to 1 second... */
		t1 = get_tbl();
		if (ppc_md.nvram_read_val(MK48T59_RTC_SECONDS) != sec) {
			break;
		}
	}

	sec = ppc_md.nvram_read_val(MK48T59_RTC_SECONDS);
	for (i = 0 ; i < 10000000 ; i++)	{ /* Should take up 1 second... */
		freq = get_tbl()-t1;
		if (ppc_md.nvram_read_val(MK48T59_RTC_SECONDS) != sec)
			break;
	}

	printk("time_init: decrementer frequency = %lu.%.6lu MHz\n",
		 freq/1000000, freq%1000000);
	tb_ticks_per_jiffy = freq / HZ;
	tb_to_us = mulhwu_scale_factor(freq, 1000000);
}
#endif /* defined (CONFIG_EVAL5200) */

static int
local_show_cpuinfo(struct seq_file *m)
{
#if defined (CONFIG_EVAL5200)
	seq_printf(m, "machine\t\t: EMK TOP5200 on EVAL5200\n");
#elif defined (CONFIG_MINI5200)
	seq_printf(m, "machine\t\t: EMK TOP5200 on MINI5200\n");
#elif defined (CONFIG_LITE5200)
	seq_printf(m, "machine\t\t: EMK LITE5200\n");
#else
	seq_printf(m, "machine\t\t: EMK TOP5200 on <unknown>\n");
#endif

	return 0;
}

static void __init
local_setup_arch(void)
{
#ifdef CONFIG_BLK_DEV_INITRD
	if (initrd_start)
		ROOT_DEV = MKDEV(RAMDISK_MAJOR, 0); /* /dev/ram */
	else
#endif
#ifdef CONFIG_ROOT_NFS
		ROOT_DEV = to_kdev_t(0x00ff); /* /dev/nfs pseudo device */
#else
		ROOT_DEV = to_kdev_t(0x0301); /* /dev/hda1 */
#endif

#ifdef CONFIG_DUMMY_CONSOLE
	conswitchp = &dummy_con;
#endif

#if defined(CONFIG_PCI) && defined(CONFIG_MPC5200)
	mpc5xxx_find_bridges();
#endif
}

#if !defined (CONFIG_EVAL5200)
/* EVAL5200 has an M48T08 Timekeeper/NVRAM - no need for this function */
/* The decrementer counts at the system (internal) clock frequency
 * divided by four.
 */
static void __init
local_calibrate_decr(void)
{
	int freq, divisor;
#ifdef CONFIG_UBOOT
	bd_t *bd = (bd_t *)__res;
	freq = bd->bi_busfreq;
#else
	/* Decrementer driven from XLB clock; CPU clock/3.5 on IceCube */
	freq = CONFIG_PPC_5xxx_PROCFREQ/3.5;
#endif
	divisor = 4;
	tb_ticks_per_jiffy = freq / HZ / divisor;
	tb_to_us = mulhwu_scale_factor(freq / divisor, 1000000);
}
#endif /* !defined (CONFIG_EVAL5200) */

void __init
platform_init(unsigned long r3, unsigned long r4, unsigned long r5,
	  unsigned long r6, unsigned long r7)
{
	parse_bootinfo(find_bootinfo());

#ifdef CONFIG_UBOOT
	if ( r3 )
		memcpy( (void *)__res,(void *)(r3+KERNELBASE), sizeof(bd_t) );

#ifdef CONFIG_BLK_DEV_INITRD
	/* take care of initrd if we have one */
	if ( r4 ) {
		initrd_start = r4 + KERNELBASE;
		initrd_end = r5 + KERNELBASE;
	}
#endif /* CONFIG_BLK_DEV_INITRD */
	/* take care of cmd line */
	if ( r6 ) {
		*(char *)(r7+KERNELBASE) = 0;
		strcpy(cmd_line, (char *)(r6+KERNELBASE));
	}
#endif /* CONFIG_UBOOT */

	mpc5xxx_set_bat();

	isa_io_base		= MPC5xxx_ISA_IO_BASE;
	isa_mem_base		= MPC5xxx_ISA_MEM_BASE;
	pci_dram_offset		= MPC5xxx_PCI_DRAM_OFFSET;

	ppc_md.setup_arch	= local_setup_arch;
	ppc_md.show_cpuinfo	= local_show_cpuinfo;
	ppc_md.show_percpuinfo	= NULL;
	ppc_md.init_IRQ		= mpc5xxx_init_irq;
	ppc_md.get_irq		= mpc5xxx_get_irq;

	ppc_md.find_end_of_memory = mpc5xxx_find_end_of_memory;
	ppc_md.setup_io_mappings = mpc5xxx_map_io;

	ppc_md.restart		= mpc5xxx_restart;
	ppc_md.power_off	= mpc5xxx_power_off;
	ppc_md.halt		= mpc5xxx_halt;

#if defined (CONFIG_EVAL5200)
/* EVAL5200 has an M48T08 Timekeeper/NVRAM */
	ppc_md.nvram_read_val	= local_nvram_read_val;
	ppc_md.nvram_write_val	= local_nvram_write_val;
	ppc_md.set_rtc_time	= local_set_rtc_time;
	ppc_md.get_rtc_time	= local_get_rtc_time;
	ppc_md.calibrate_decr	= local_calibrate_decr;
	ppc_md.time_init	= local_init;
#else
	/* MPC5xxx have no timekeeper part */
	ppc_md.time_init	= NULL;
	ppc_md.get_rtc_time	= NULL;
	ppc_md.set_rtc_time	= NULL;
	ppc_md.calibrate_decr	= local_calibrate_decr;
#endif

#ifdef  CONFIG_SERIAL_TEXT_DEBUG
	ppc_md.progress = mpc5xxx_progress;
#endif  /* CONFIG_SERIAL_TEXT_DEBUG */

#ifdef CONFIG_VT
	ppc_md.kbd_setkeycode	= mpc5xxx_setkeycode;
	ppc_md.kbd_getkeycode	= mpc5xxx_getkeycode;
	ppc_md.kbd_translate	= mpc5xxx_translate;
	ppc_md.kbd_unexpected_up= mpc5xxx_unexpected_up;
	ppc_md.kbd_leds		= NULL;
	ppc_md.kbd_init_hw	= mpc5xxx_kbd_init_hw;
#ifdef CONFIG_MAGIC_SYSRQ
	ppc_md.ppc_kbd_sysrq_xlate	= mpc5xxx_sysrq_xlate;
	SYSRQ_KEY = 0x54;
#endif
#endif	/* CONFIG_VT */
	
#ifdef CONFIG_BLK_DEV_IDE_MPC5xxx
	ppc_ide_md.ide_init_hwif	= mpc5xxx_ide_init_hwif_ports;
	ppc_ide_md.default_io_base	= mpc5xxx_ide_get_base;
#endif /* CONFIG_BLK_DEV_IDE_MPC5xxx */

	mpc5xxx_bestcomm_init();
}
