/*
 * arch/ppc/platforms/tqm5200.h
 *
 * Definitions for TQ-Systems TQM5200 MPC5xxx Standard Development
 * Platform board support
 *
 * Modification of Dale Farnsworth's <dale.farnsworth@mvista.com>
 * version made by Kent Borg.
 *
 * Copyright 2003 Motorola Inc.
 * Copyright 2003 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */
 /*
 * From Processor to PCI:
 *   PCI Mem Space: 0x40000000 - 0x50000000 -> 0x40000000 - 0x50000000 (256 MB)
 *   PCI I/O Space: 0x50000000 - 0x51000000 -> 0x50000000 - 0x51000000 (16 MB)
 *
 * From PCI to Processor:
 *   System Memory: 0x00000000 -> 0x00000000
 */


#ifndef __PLATFORMS_TQM5200_H
#define __PLATFORMS_TQM5200_H

#define MPC5xxx_PROC_PCI_MEM_START	0x40000000U
#define MPC5xxx_PROC_PCI_MEM_END	0x4fffffffU
#define MPC5xxx_PCI_MEM_START		0x40000000U
#define MPC5xxx_PCI_MEM_END		0x4fffffffU
#define MPC5xxx_PCI_MEM_SIZE		(MPC5xxx_PCI_MEM_END + 1 - MPC5xxx_PCI_MEM_START)

#define MPC5xxx_PCI_MEM_VIRT_START	0x40000000U

#define MPC5xxx_PROC_PCI_IO_START	0x50000000U
#define MPC5xxx_PROC_PCI_IO_END		0x50ffffffU
#define MPC5xxx_PCI_IO_START		0x50000000U
#define MPC5xxx_PCI_IO_END		0x50ffffffU
#define MPC5xxx_PCI_IO_SIZE		(MPC5xxx_PCI_IO_END + 1 - MPC5xxx_PCI_IO_START)

#define MPC5xxx_PCI_IO_VIRT_START	0x50000000U

#define MPC5xxx_PCI_DRAM_OFFSET		0x00000000U
#define MPC5xxx_PCI_PHY_MEM_OFFSET	0x00000000U

#define MPC5xxx_ISA_IO_BASE		MPC5xxx_PROC_PCI_IO_START
#define MPC5xxx_ISA_MEM_BASE		MPC5xxx_PROC_PCI_MEM_START

#ifdef CONFIG_PS2MULT
#define RS_TABLE_SIZE  4
#else
#define RS_TABLE_SIZE  3
#endif

#ifdef CONFIG_UBOOT
#define	MPC5xxx_INIT_BAUD_BASE		0
#else
#define	MPC5xxx_INIT_BAUD_BASE		CONFIG_PPC_5xxx_IPBFREQ
#endif

#define CFG_SRAM_PADDR                  0xE5000000
#define CFG_SRAM_SIZE                   0x00080000

#define STD_PSC_OP(num)					\
{							\
	.magic = 0,					\
	.baud_base = MPC5xxx_INIT_BAUD_BASE,		\
	.irq = MPC5xxx_PSC##num##_IRQ,			\
	.flags = ASYNC_BOOT_AUTOCONF|ASYNC_SKIP_TEST, 	\
	.iomem_base = (void *)MPC5xxx_PSC##num,		\
	.io_type = SERIAL_IO_MEM,			\
},

#ifdef CONFIG_PS2MULT
#define SERIAL_PORT_DFNS	\
	STD_PSC_OP(1)		\
	STD_PSC_OP(2)		\
	STD_PSC_OP(3)		\
{							\
	.magic = 0,					\
	.baud_base = MPC5xxx_INIT_BAUD_BASE,		\
	.irq = MPC5xxx_IRDA_IRQ,			\
	.flags = ASYNC_BOOT_AUTOCONF|ASYNC_SKIP_TEST, 	\
	.iomem_base = (void *)MPC5xxx_PSC6,		\
	.io_type = SERIAL_IO_MEM,			\
}
#else 
#define SERIAL_PORT_DFNS	\
	STD_PSC_OP(1)		\
	STD_PSC_OP(2)		\
	STD_PSC_OP(3)
#endif /* CONFIG_PS2MULT */

#endif /* __PLATFORMS_TQM5200_H */
