/***********************************************************************
 *
 * (C) Copyright 2005
 * DENX Software Engineering
 * Wolfgang Denk, wd@denx.de
 * All rights reserved.
 *
 ***********************************************************************/

#include <linux/version.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/poll.h>

#include <linux/proc_fs.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/inka_digio.h>

#include <asm/types.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <asm/mpc5xxx.h>


#define INKA_DIGIO_VERSION "$Revision: 1824 $"
#define INKA_DIGIO_NAME "inka_digio"

#ifndef INKA_DIGIO_MAJOR
#define INKA_DIGIO_MAJOR	248
#endif

#define INKA_DIGIN_MINOR	0
#define INKA_DIGIO_MINOR	1
#define INKA_DIGIO_DIMM_MINOR	2
#define INKA_DIGIO_MINOR_MAX	2


#define CFG_GPIO_IN_BASE  0x30400000 /* Configured by u-boot */
static u_long gpio_base;

#if INKA_DIGIO_MAJOR == 0
static int inka_digio_major;
#endif

static int inka_digin_proc_read(char *page, char **start,
				   off_t off, int count,
				   int *eof, void *data);
static int inka_digio_leds_proc_read(char *page, char **start,
				   off_t off, int count,
				   int *eof, void *data);
static int inka_digio_leds_proc_write(struct file *file, const char *buffer,
				    u_long count, void *data);

static int inka_digio_drawer_proc_read(char *page, char **start,
				   off_t off, int count,
				   int *eof, void *data);
static int inka_digio_drawer_proc_write(struct file *file, const char *buffer,
				    u_long count, void *data);

static int inka_digio_dir_proc_read(char *page, char **start,
				   off_t off, int count,
				   int *eof, void *data);
static int inka_digio_dir_proc_write(struct file *file, const char *buffer,
				    u_long count, void *data);

static int inka_digio_inv_proc_read(char *page, char **start,
				   off_t off, int count,
				   int *eof, void *data);
static int inka_digio_inv_proc_write(struct file *file, const char *buffer,
				    u_long count, void *data);

static int inka_digio_dimm_proc_read(char *page, char **start,
				   off_t off, int count,
				   int *eof, void *data);
static int inka_digio_dimm_proc_write(struct file *file, const char *buffer,
				    u_long count, void *data);

static int read_state(const char *buffer, int nbits, u_int *retval);
/*
 * MPCIO_8XX digital I/O operations
 */
static u_int inka_digin_get_input(void)
{
	u_int status;

	status =
		(*(volatile u_char *)(gpio_base + 0) & 0xff) << 0 |
		(*(volatile u_char *)(gpio_base + 1) & 0xff) << 8 |
		(*(volatile u_char *)(gpio_base + 2) & 0xff) << 16 |
		(*(volatile u_char *)(gpio_base + 3) & 0xff) << 24;
	return status;
}

static u_int inka_digio_get_output(void)
{
	u_int state;
	struct mpc5xxx_gpio *gpio = (struct mpc5xxx_gpio *)MPC5xxx_GPIO;


	state =
		/* GPT 0-5 -LEDS */
		(((*(volatile u_long *)MPC5xxx_GPT0_STATUS & 0x100) >> 8)
		 << (ffs(DIGIO_LED0) - 1)) |
		(((*(volatile u_long *)MPC5xxx_GPT1_STATUS & 0x100) >> 8)
		 << (ffs(DIGIO_LED1) - 1)) |
		(((*(volatile u_long *)MPC5xxx_GPT2_STATUS & 0x100) >> 8)
		 << (ffs(DIGIO_LED2) - 1)) |
		(((*(volatile u_long *)MPC5xxx_GPT3_STATUS & 0x100) >> 8)
		 << (ffs(DIGIO_LED3) - 1)) |
		(((*(volatile u_long *)MPC5xxx_GPT4_STATUS & 0x100) >> 8)
		 << (ffs(DIGIO_LED4) - 1)) |
		(((*(volatile u_long *)MPC5xxx_GPT5_STATUS & 0x100) >> 8)
		 << (ffs(DIGIO_LED5) - 1)) |

		/* PSC3_6,7 - cash drawer 1,2 */
		(((gpio->simple_ival & 0x00003000) >> 12) << (ffs(DIGIO_DRAWER1) - 1)) |
		/* PSC3_8 - remote on/off */
		(((gpio->sint_dvo & 0x04) >> 2) << (ffs(DIGIO_INV) - 1)) |
		/* GPIO_WKUP6,7 - LCD Up/Down and Left/Right */
		(((*(volatile u_long *)MPC5xxx_WU_GPIO_DATAOUT & 0xc0000000) >> 30)
		 << (ffs(DIGIO_LCD_UD) - 1));

	return state;
}

static void inka_digio_set_output(u_int state)
{
	struct mpc5xxx_gpio *gpio = (struct mpc5xxx_gpio *)MPC5xxx_GPIO;
	u_long val;

	/* Bits 0-5,7 to GPT 0-5,7 */
	if (state & DIGIO_LED0)
		*(volatile u_long *)MPC5xxx_GPT0_ENABLE |= 0x10;
	else
		*(volatile u_long *)MPC5xxx_GPT0_ENABLE &= ~0x10;

	if (state & DIGIO_LED1)
		*(volatile u_long *)MPC5xxx_GPT1_ENABLE |= 0x10;
	else
		*(volatile u_long *)MPC5xxx_GPT1_ENABLE &= ~0x10;

	if (state & DIGIO_LED2)
		*(volatile u_long *)MPC5xxx_GPT2_ENABLE |= 0x10;
	else
		*(volatile u_long *)MPC5xxx_GPT2_ENABLE &= ~0x10;

	if (state & DIGIO_LED3)
		*(volatile u_long *)MPC5xxx_GPT3_ENABLE |= 0x10;
	else
		*(volatile u_long *)MPC5xxx_GPT3_ENABLE &= ~0x10;

	if (state & DIGIO_LED4)
		*(volatile u_long *)MPC5xxx_GPT4_ENABLE |= 0x10;
	else
		*(volatile u_long *)MPC5xxx_GPT4_ENABLE &= ~0x10;

	if (state & DIGIO_LED5)
		*(volatile u_long *)MPC5xxx_GPT5_ENABLE |= 0x10;
	else
		*(volatile u_long *)MPC5xxx_GPT5_ENABLE &= ~0x10;


	/* Bits 8,9,10 to GPS */
	if (state & DIGIO_DRAWER1)
		gpio->simple_dvo |= 0x1000;
	else
		gpio->simple_dvo &= ~0x1000;

	if (state & DIGIO_DRAWER2)
		gpio->simple_dvo |= 0x2000;
	else
		gpio->simple_dvo &= ~0x2000;


	if (state & DIGIO_INV)
		gpio->sint_dvo |= 0x04;
	else
		gpio->sint_dvo &= ~0x04;

	/* bits 11,12,13 to wakeup GPIO */
	val = *(volatile u_long *)MPC5xxx_WU_GPIO_DATAOUT;

	if (state & DIGIO_LCD_UD)
		val |= 0x40000000;
	else
		val &= ~0x40000000;

	if (state & DIGIO_LCD_LR)
		val |= 0x80000000;
	else
		val &= ~0x80000000;

	*(volatile u_long *)MPC5xxx_WU_GPIO_DATAOUT = val;
}

static u_int inka_digio_get_dimm(void)
{
	u_int ret;

	ret = (*(volatile ulong *)MPC5xxx_GPT7_PWMCFG) >> 16;

	return ret;
}


static u_int inka_digio_set_dimm(u_int val)
{
	if (val > 255)
		val = 255;

	*(volatile ulong *)MPC5xxx_GPT7_PWMCFG = (val << 16) | 1;

	return 0;
}
/*
 * device file operations
 */

static ssize_t inka_digio_read(struct file *file, char *buf, size_t bufsz, loff_t *off)
{
	ssize_t	       ret   = -EINVAL;
	u_int		state = 0U;

	switch (MINOR(file->f_dentry->d_inode->i_rdev)) {
	case INKA_DIGIN_MINOR:
		/* check pins' state */
		state = inka_digin_get_input();
		break;

	case INKA_DIGIO_MINOR:
		/* return latched output pins' state */
		state = inka_digio_get_output();
		break;

	case INKA_DIGIO_DIMM_MINOR:
		state = inka_digio_get_dimm();
		break;

	default:
		ret = -ENXIO;
		goto error;
	}

	if (put_user(state, (int *)buf)) {
		ret = -EFAULT;
		goto error;
	}

	ret = sizeof(state);
error:
	return ret;
}

static ssize_t inka_digio_write(struct file *file, const char *buf, size_t bufsz, loff_t *off)
{
	ssize_t		   ret	 = -EINVAL;
	u_int		    state = 0U;

	switch (MINOR(file->f_dentry->d_inode->i_rdev)) {
	case INKA_DIGIO_MINOR:
		/* copy state from user */
		if (get_user(state, (int *)buf)) {
			ret = -EFAULT;
			goto error;
		}

		/* set current output pins' state */
		inka_digio_set_output(state);
		break;

	case INKA_DIGIO_DIMM_MINOR:
		/* copy state from user */
		if (get_user(state, (int *)buf)) {
			ret = -EFAULT;
			goto error;
		}

		/* set current output pins' state */
		inka_digio_set_dimm(state);
		break;

	default:
		ret = -ENXIO;
		goto error;
	}

	ret = sizeof(state);
error:
	return ret;
}

static u_int inka_digio_poll(struct file *file, struct poll_table_struct *ptbl)
{
	u_int ret = POLLNVAL;

	switch (MINOR(file->f_dentry->d_inode->i_rdev)) {
	case INKA_DIGIN_MINOR:
	case INKA_DIGIO_MINOR:
	case INKA_DIGIO_DIMM_MINOR:
		ret = POLLIN | POLLRDNORM;
		break;
	default:
		goto error;
	}

error:
	return ret;
}

static int inka_digio_open(struct inode *inode, struct file *file)
{
	if (MINOR(inode->i_rdev) > INKA_DIGIO_MINOR_MAX)
		return -ENODEV;
	return 0;
}


static struct file_operations inka_digio_fops = {
	read:  inka_digio_read,
	write: inka_digio_write,
	poll:  inka_digio_poll,
	open:  inka_digio_open,
};


static int __init inka_digio_init(void)
{
	int			ret   = -ENODEV;
#ifdef CONFIG_PROC_FS
	struct proc_dir_entry * proc;
#endif

	gpio_base = (u_long)ioremap(CFG_GPIO_IN_BASE, 0x100);

	ret = devfs_register_chrdev(INKA_DIGIO_MAJOR, INKA_DIGIO_NAME,
				     &inka_digio_fops);
	if (ret < 0) {
		goto error;
	}
#if INKA_DIGIO_MAJOR == 0
	inka_digio_major = ret;
#endif

#ifdef CONFIG_PROC_FS
	if (!(proc = create_proc_read_entry(DIGI_IN_PROC,
					     S_IFREG|S_IRUGO, NULL,
					     inka_digin_proc_read, NULL))) {
		printk(KERN_ERR "failed to create /proc/" DIGI_IN_PROC "\n");
		goto error;
	}

	if (!proc_mkdir(DIGI_OUT_PROC, NULL)) {
		printk(KERN_ERR "failed to create /proc/" DIGI_OUT_PROC "\n");
		goto error;
	}

	if (!(proc = create_proc_entry(DIGI_OUT_LEDS_PROC, S_IFREG|S_IRUGO, NULL))) {
		printk(KERN_ERR "failed to create /proc/" DIGI_OUT_LEDS_PROC "\n");
		goto error;
	}

	proc->read_proc	 = inka_digio_leds_proc_read;
	proc->write_proc = inka_digio_leds_proc_write;

	if (!(proc = create_proc_entry(DIGI_OUT_DRAWER_PROC, S_IFREG|S_IRUGO, NULL))) {
		printk(KERN_ERR "failed to create /proc/" DIGI_OUT_DRAWER_PROC "\n");
		goto error;
	}

	proc->read_proc	 = inka_digio_drawer_proc_read;
	proc->write_proc = inka_digio_drawer_proc_write;

	if (!(proc = create_proc_entry(DIGI_OUT_DIR_PROC, S_IFREG|S_IRUGO, NULL))) {
		printk(KERN_ERR "failed to create /proc/" DIGI_OUT_DIR_PROC "\n");
		goto error;
	}

	proc->read_proc	 = inka_digio_dir_proc_read;
	proc->write_proc = inka_digio_dir_proc_write;

	if (!(proc = create_proc_entry(DIGI_OUT_INV_PROC, S_IFREG|S_IRUGO, NULL))) {
		printk(KERN_ERR "failed to create /proc/" DIGI_OUT_INV_PROC "\n");
		goto error;
	}

	proc->read_proc	 = inka_digio_inv_proc_read;
	proc->write_proc = inka_digio_inv_proc_write;

	if (!(proc = create_proc_entry(DIGI_OUT_DIMM_PROC, S_IFREG|S_IRUGO, NULL))) {
		printk(KERN_ERR "failed to create /proc/" DIGI_OUT_DIMM_PROC "\n");
		goto error;
	}

	proc->read_proc	 = inka_digio_dimm_proc_read;
	proc->write_proc = inka_digio_dimm_proc_write;


#endif

	printk(KERN_INFO "INKA Digital I/O driver\n");


	ret = 0;
error:
	return ret;
}

static void __devexit inka_digio_cleanup(void)
{

	iounmap((void *)gpio_base);
#ifdef CONFIG_PROC_FS
	remove_proc_entry(DIGI_IN_PROC,	 NULL);
	remove_proc_entry(DIGI_OUT_PROC, NULL);
#endif

#if INKA_DIGIO_MAJOR == 0
	devfs_unregister_chrdev(inka_digio_major, INKA_DIGIO_NAME);
#else
	devfs_unregister_chrdev(INKA_DIGIO_MAJOR, INKA_DIGIO_NAME);
#endif
}

/*
 * procfs operations
 */
#ifdef CONFIG_PROC_FS
static int dump_state_out(u_int state, u_char *buf, int nbits)
{
	int i;
	int ret;
	u_char *out = buf;

	for (i = 0; i < nbits; i++) {
		ret = sprintf(out, "%d ", (state & (1 << i)) >> i);
		out += ret;
	}
	ret = sprintf(out, "\n");
	out += ret;

	return (out - buf);
}

static int dump_state_in(u_int state, u_char *buf)
{
	int ret = -1;
	u_char *out = buf;

	ret = sprintf(out, "Display type: %#x\n", state & DIGIN_DISPLAY_TYPE_MASK);
	out += ret;

	ret = sprintf(out, "Version: %#x\n",
		(state & DIGIN_VERSION_MASK) >> (ffs(DIGIN_VERSION_MASK) - 1));
	out += ret;

	ret = sprintf(out, "Touchscreen type: %#x\n",
		(state & DIGIN_TOUCHSCR_MASK) >> (ffs(DIGIN_TOUCHSCR_MASK) - 1));
	out += ret;

	ret = sprintf(out, "Keyboard type: %#x\n",
		(state & DIGIN_KEYB_MASK) >> (ffs(DIGIN_KEYB_MASK) - 1));
	out+= ret;

	ret = sprintf(out, "Printer: %d\n",
		(state & DIGIN_PRINTER) >> (ffs(DIGIN_PRINTER) - 1));
	out += ret;

	ret = sprintf(out, "Battery low: %d\n",
		(state & DIGIN_BATTERY_LOW) >> (ffs(DIGIN_BATTERY_LOW) - 1));
	out += ret;

	ret = sprintf(out, "Drawer switch 1: %d\n",
		(state & DIGIN_DRAWER_SW1) >> (ffs(DIGIN_DRAWER_SW1) - 1));
	out += ret;

	ret = sprintf(out, "Drawer switch 2: %d\n",
		(state & DIGIN_DRAWER_SW2) >> (ffs(DIGIN_DRAWER_SW2) - 1));
	out += ret;

	ret = sprintf(out, "CompactFlash slave CD1: %d\n",
		(state & DIGIN_CFLASH_CD1) >> (ffs(DIGIN_CFLASH_CD1) - 1));
	out += ret;

	ret = sprintf(out, "CompactFlash slave CD2: %d\n",
		(state & DIGIN_CFLASH_CD2) >> (ffs(DIGIN_CFLASH_CD2) - 1));
	out += ret;

	ret = sprintf(out, "Memory ext 1: %d\n",
		(state & DIGIN_EXT_MEM1) >> (ffs(DIGIN_EXT_MEM1) - 1));
	out += ret;

	ret = sprintf(out, "Memory ext 2: %d\n",
		(state & DIGIN_EXT_MEM2) >> (ffs(DIGIN_EXT_MEM2) - 1));
	out += ret;

	return (out - buf);
}

static int inka_digin_proc_read(char *page, char **start,
				   off_t off, int count,
				   int *eof, void *data)
{
	int    ret;
	u_int state;

	state = inka_digin_get_input();
	ret  = dump_state_in(state, page);

	return ret;
}

static int inka_digio_leds_proc_read(char *page, char **start, off_t off,
					int count, int *eof, void *data)
{
	int    ret;
	u_int state;

	state = inka_digio_get_output();
	ret   = dump_state_out(state, page, 6);

	return ret;
}

static int inka_digio_leds_proc_write(struct file *file, const char *buffer,
				   u_long count, void *data)
{
	char buf[128];
	int ret;
	int val;


	if (copy_from_user(buf, buffer, sizeof(buf))) {
		ret = -EFAULT;
		goto error;
	}

	if (read_state(buf, 6, &ret)) {
		ret = -EINVAL;
		goto error;
	}

	val = inka_digio_get_output();
	val &= ~DIGIO_LEDS_MASK;
	inka_digio_set_output(val | ret);

	ret = count;
 error:
	return ret;
}


static int inka_digio_drawer_proc_read(char *page, char **start, off_t off,
					int count, int *eof, void *data)
{
	int    ret;
	u_int state;

	state = inka_digio_get_output();
	ret = dump_state_out((state & DIGIO_DRAWER_MASK) >>
				(ffs(DIGIO_DRAWER1) - 1), page, 2);

	return ret;
}

static int inka_digio_drawer_proc_write(struct file *file, const char *buffer,
				   u_long count, void *data)
{
	char buf[16];
	int ret;
	int val;


	if (copy_from_user(buf, buffer, sizeof(buf))) {
		ret = -EFAULT;
		goto error;
	}

	if (read_state(buf, 2, &ret)) {
		ret = -EINVAL;
		goto error;
	}

	val = inka_digio_get_output();
	val &= ~DIGIO_DRAWER_MASK;
	inka_digio_set_output(val | (ret << (ffs(DIGIO_DRAWER1) - 1)));
	ret = count;
 error:
	return ret;
}


static int inka_digio_inv_proc_read(char *page, char **start, off_t off,
					int count, int *eof, void *data)
{
	int    ret;
	u_int state;

	state = inka_digio_get_output();
	ret   = dump_state_out((state & DIGIO_INV) >> (ffs(DIGIO_INV) - 1),
						   page, 1);

	return ret;
}

static int inka_digio_inv_proc_write(struct file *file, const char *buffer,
				   u_long count, void *data)
{
	char buf[16];
	int ret;
	int val;


	if (copy_from_user(buf, buffer, sizeof(buf))) {
		ret = -EFAULT;
		goto error;
	}

	if (read_state(buf, 1, &ret)) {
		ret = -EINVAL;
		goto error;
	}

	val = inka_digio_get_output();
	val &= ~DIGIO_INV;

	inka_digio_set_output(val | (ret << (ffs(DIGIO_INV) - 1)));
	ret = count;
 error:
	return ret;
}


static int inka_digio_dir_proc_read(char *page, char **start, off_t off,
					int count, int *eof, void *data)
{
	int    ret;
	u_int state;

	state = inka_digio_get_output();

	ret   = dump_state_out((state & (DIGIO_LCD_UD|DIGIO_LCD_LR)) >>
			 (ffs(DIGIO_LCD_UD) - 1), page, 2);

	return ret;
}

static int inka_digio_dir_proc_write(struct file *file, const char *buffer,
				   u_long count, void *data)
{
	char buf[16];
	int ret;
	int val;


	if (copy_from_user(buf, buffer, sizeof(buf))) {
		ret = -EFAULT;
		goto error;
	}

	if (read_state(buf, 2, &ret)) {
		ret = -EINVAL;
		goto error;
	}

	val = inka_digio_get_output();
	val &= ~(DIGIO_LCD_UD | DIGIO_LCD_LR);
	inka_digio_set_output(val | (ret << (ffs(DIGIO_LCD_UD) - 1)));
	ret = count;
 error:
	return ret;
}


static int inka_digio_dimm_proc_read(char *page, char **start, off_t off,
					int count, int *eof, void *data)
{
	int    ret;
	u_int state;

	state = inka_digio_get_dimm();
	ret = sprintf(page, "%d\n", state);
	
	return ret;
}

static int inka_digio_dimm_proc_write(struct file *file, const char *buffer,
				   u_long count, void *data)
{
	char buf[16];
	int ret;


	if (copy_from_user(buf, buffer, sizeof(buf))) {
		ret = -EFAULT;
		goto error;
	}

	if (sscanf(buf, "%d", &ret) != 1) {
		ret = -EINVAL;
		goto error;
	}

	inka_digio_set_dimm(ret);

	ret = count;
 error:
	return ret;
}


static int read_state(const char *buffer, int nbits, u_int *retval)
{
	char fmt[128];
	int bits[32];
	char *ptr;
	int i, rc;

	for (i = 0, ptr = fmt; i < nbits; i++) {
		rc = sprintf(ptr, "%%u ");
		ptr += rc;
	}

	if ((rc = sscanf(buffer, fmt,
				&bits[0], &bits[1],
				&bits[2], &bits[3],
				&bits[4], &bits[5],
				&bits[6], &bits[7],
				&bits[8], &bits[9],
				&bits[10], &bits[11],
				&bits[12], &bits[13])) != nbits) {
		rc = -EINVAL;
		goto error;
	}

	rc = 0;
	*retval = 0;
	for(i = 0; i < nbits; i++) {
		*retval |= (bits[i] << i);
	}

error:
	return rc;
}
#endif

EXPORT_NO_SYMBOLS;

module_init(inka_digio_init)
module_exit(inka_digio_cleanup)
