/*
 *  drivers/char/m41t00.c
 *
 *  Copyright (C) 2003 DAVE Srl (info@wawnet.biz)
 *
 *  Derived from drivers/mtd/nand/pcf8563.c
 *
 *
 * $Id: m41t00.c 1824 2005-05-10 07:30:07Z hg $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Overview:
 * This is a device driver for the ST M41T00 RTC situated
 * on DAVE Srl - PPChameleon-EVB Board.
 * This version changes from 1.0 (Pelk 1.2.0 kernel-2.4.20)
 * - i2c_probe function is not called.
 *
*/

/*
 * Standard in kernel modules
 */
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/module.h>
#include <linux/miscdevice.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/rtc.h>
#include <linux/smp_lock.h>
#include <linux/version.h>
#include <linux/i2c.h>
#include <linux/slab.h>
#include <linux/string.h>
#include <linux/init.h>
#include <asm/types.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/time.h>
#include <asm/machdep.h>	/* ppc_md */
#include <asm/semaphore.h>



#ifndef KERNEL_VERSION
# define KERNEL_VERSION(a,b,c)	(((a) << 16) + ((b) << 8) + (c))
#endif

#if (!defined(LINUX_VERSION_CODE) || LINUX_VERSION_CODE < KERNEL_VERSION(2,2,0))
#error "You need to use at least Linux kernel version 2.2.0"
#endif


/***************************************************************************
 * Global stuff
 ***************************************************************************
*/

/* #define	DEBUG	1	*/

/*
 * Deal with CONFIG_MODVERSIONS
 */

/*
#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
# include <linux/modversions.h>
#endif
*/

/*
 * For character devices
 */
#include <linux/fs.h>		/* character device definitions */
#include <linux/wrapper.h>	/* wrapper for compatibility with future versions */


/*
 * Device Declarations
 */

/*
 * The name for our device, as it will appear in /proc/devices
 */
#define DEVICE_NAME	"rtc"
#define	RTC_VERSION	"$Revision: 2.0"

/***************************************************************************
 * Local stuff
 ***************************************************************************
 */

#ifdef	DEBUG
# define debugk(fmt,args...)	printk(fmt ,##args)
# define DEBUG_OPTARG(arg)	arg,
#else
# define debugk(fmt,args...)
# define DEBUG_OPTARG(arg)
#endif


static DECLARE_MUTEX (mutex);
static struct i2c_driver m41t00_driver;

/*
static unsigned short ignore[] = { I2C_CLIENT_END };
static unsigned short normal_addr[] = { 0x68, I2C_CLIENT_END };

static struct i2c_client_address_data addr_data = {
	normal_i2c:		normal_addr,
	normal_i2c_range:	ignore,
	probe:			ignore,
	probe_range:		ignore,
	ignore:			ignore,
	ignore_range:		ignore,
	force:			ignore,
};
*/
/*
 * Prototypes
 */

static int m41t00_rtc_open (struct inode *, struct file *);
static int m41t00_rtc_release (struct inode *, struct file *);
static int m41t00_rtc_ioctl (struct inode *, struct file *, unsigned int,
			     unsigned long);
static unsigned int bcd2bin (unsigned int);
static unsigned int bin2bcd (unsigned int);
static unsigned char rtc_rd (unsigned char);
static void rtc_wr (unsigned char, unsigned char);
static unsigned long m41t00_get_rtc_time (void);
static int m41t00_set_rtc_time (unsigned long now);
static void m41t00_reset_rtc (void);
static void rtcset_now_pointers (void);

/* module functions */
int init_module (void);
void cleanup_module (void);

/* avoid multiple access */
static char rtc_status = 0;

/* save/restore old machdep pointers */
int (*save_set_rtc_time) (unsigned long);
unsigned long (*save_get_rtc_time) (void);

/***************************************************************************
 * I2C stuff
 ***************************************************************************
 */
#define DAT(x) ((unsigned int)(x->data))

static struct i2c_client *clnt = NULL;

static int
m41t00_attach (struct i2c_adapter *adap, int addr, unsigned short flags,
	       int kind)
{
	clnt = kmalloc (sizeof (*clnt), GFP_KERNEL);

	if (!clnt) {
		return -ENOMEM;
	}

	strcpy (clnt->name, "M41T00");
	clnt->id = m41t00_driver.id;
	clnt->flags = 0;
	clnt->addr = addr;
	clnt->adapter = adap;
	clnt->driver = &m41t00_driver;
	clnt->data = NULL;

	return i2c_attach_client (clnt);
}

static int m41t00_probe (struct i2c_adapter *adap)
{

	m41t00_attach (adap, 0x68, 0, 0);
	/*
	 * Set system time
	 * Code copied from arch/ppc/kernel/time.c
	 */
	rtcset_now_pointers ();

	return 0;

}

static int m41t00_detach (struct i2c_client *client)
{
	i2c_detach_client (client);
	return 0;
}

/*
static int m41t00_command (struct i2c_client *client,unsigned int cmd,void *arg)
{
	return 0;
}
*/

static struct i2c_driver m41t00_driver = {
	name:"M41T00",
	id:I2C_DRIVERID_STM41T00,
	flags:I2C_DF_NOTIFY,
	attach_adapter:m41t00_probe,
	detach_client:m41t00_detach,
};

/***************************************************************************
 * Driver interface
 ***************************************************************************
 */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0)

static struct file_operations m41t00_rtc_fops = {
	NULL,			/* lseek        */
	NULL,			/* read         */
	NULL,			/* write        */
	NULL,			/* readdir      */
	NULL,			/* select       */
	m41t00_rtc_ioctl,	/* ioctl        */
	NULL,			/* mmap         */
	m41t00_rtc_open,	/* open         */
	NULL,			/* flush        */
	m41t00_rtc_release	/* close        */
};

#else  /* Linux kernel >= 2.3.x */

static struct file_operations m41t00_rtc_fops = {
	owner:THIS_MODULE,
	open:m41t00_rtc_open,
	release:m41t00_rtc_release,
	ioctl:m41t00_rtc_ioctl,
};
#endif

/*
 * We sponge a minor off of the misc major. No need slurping
 * up another valuable major dev number for this.
 */
static struct miscdevice rtc_dev = {
	RTC_MINOR,
	DEVICE_NAME,
	&m41t00_rtc_fops,
};


/***************************************************************************
 *
 * Initialize the driver - Register the character device
 */
static int __init m41t00_rtc_init (void)
{
	int err;

	printk (KERN_INFO "M41T00 Real-Time Clock Driver " RTC_VERSION "\n");

	/* misc_register(&rtc_dev); */

	err = i2c_add_driver (&m41t00_driver);
	if (err) {
		printk (KERN_INFO "Register I2C driver failed, errno is %d\n",
			err);
		return err;
	}
	/* Switching kernel RTC pointers
	   will be done into rtcset_now_pointers() function */
	err = misc_register (&rtc_dev);
	if (err) {
		printk ("m41t00.o: Register misc driver failed, errno is %d\n", err);
		err = i2c_del_driver (&m41t00_driver);
		if (err) {
			printk ("m41t00.o: Unregister I2C driver failed, "
				"errno is %d\n", err);
		}
		return err;
	}

	return 0;
}

static void rtcset_now_pointers (void)
{
	unsigned long now, flags;
	extern time_t last_rtc_update;
	extern rwlock_t xtime_lock;

	debugk ("RTC switching kernel pointers\n");

	save_set_rtc_time = ppc_md.set_rtc_time;
	save_get_rtc_time = ppc_md.get_rtc_time;

	ppc_md.set_rtc_time = m41t00_set_rtc_time;
	ppc_md.get_rtc_time = m41t00_get_rtc_time;

	m41t00_reset_rtc ();
	debugk ("RTC switching kernel pointer ends\n");

	/*
	 * Set system time
	 * Code copied from arch/ppc/kernel/time.c
	 */
	write_lock_irqsave (&xtime_lock, flags);

	now = m41t00_get_rtc_time ();
	debugk ("Set System Time from RTC Time: %lu\n", now);

	xtime.tv_usec = 0;
	if (now >= 0)
		xtime.tv_sec = now;

	last_rtc_update = now - 658;
	time_adjust = 0;	/* stop active adjtime() */
	time_status |= STA_UNSYNC;
	time_state = TIME_ERROR;
	time_maxerror = NTP_PHASE_LIMIT;
	time_esterror = NTP_PHASE_LIMIT;

	write_unlock_irqrestore (&xtime_lock, flags);

}

/***************************************************************************
 *
 * called whenever a process attempts to open the device
 */
static int m41t00_rtc_open (struct inode *inode, struct file *file)
{
	lock_kernel ();
	if (rtc_status) {
		unlock_kernel ();
		return -EBUSY;
	}

	rtc_status = 1;
	unlock_kernel ();

	MOD_INC_USE_COUNT;

	return 0;
}



/***************************************************************************
 *
 * Called when a process closes the device.
 */
static int m41t00_rtc_release (struct inode *inode, struct file *file)
{
	lock_kernel ();
	rtc_status = 0;
	unlock_kernel ();

	MOD_DEC_USE_COUNT;

	return 0;
}


/***************************************************************************
 *
 * ioctl entry point:
 */
static int m41t00_rtc_ioctl (struct inode *inode, struct file *file,
			     unsigned int cmd, unsigned long arg)
{
	struct rtc_time tm;
	int leap_yr;
	static const unsigned char days_in_month[] =
		{ 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

	switch (cmd) {
	case RTC_RD_TIME:	/* Read the time/date from RTC  */

		to_tm (m41t00_get_rtc_time (), &tm);

		/* convert from rtc_time to struct tm conventions */
		tm.tm_year -= 1900;
		tm.tm_mon -= 1;

		debugk ("RTC_RD_TIME: %4d-%02d-%02d-%02d:%02d:%02d\n",
			tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,
			tm.tm_hour, tm.tm_min, tm.tm_sec);
		debugk ("PID %d\n", current->pid);

		return (copy_to_user ((void *) arg, &tm, sizeof (tm))) ?
			-EFAULT : 0;

	case RTC_SET_TIME:	/* Set the time/date in the RTC */

		if (!capable (CAP_SYS_TIME))
			return -EACCES;

		if (copy_from_user
		    (&tm, (struct rtc_time *) arg, sizeof (tm)))
			return -EFAULT;

		/* convert from struct tm to rtc_time conventions */
		tm.tm_year += 1900;
		tm.tm_mon += 1;

		/* simple but OK for all dates in time_t range */
		leap_yr = ((tm.tm_year % 4) == 0);

		if ((tm.tm_year < 1970) || (tm.tm_mon > 12) || (tm.tm_mday == 0) ||
		    (tm.tm_mday > (days_in_month[tm.tm_mon] + ((tm.tm_mon == 2) && leap_yr))) ||
		    (tm.tm_hour >= 24) || (tm.tm_min >= 60) || (tm.tm_sec >= 60)) {
			return (-EINVAL);
		}

		debugk ("RTC_SET_TIME: %4d-%02d-%02d-%02d:%02d:%02d\n",
			tm.tm_year, tm.tm_mon, tm.tm_mday,
			tm.tm_hour, tm.tm_min, tm.tm_sec);

		m41t00_set_rtc_time (mktime
				     (tm.tm_year, tm.tm_mon, tm.tm_mday,
				      tm.tm_hour, tm.tm_min, tm.tm_sec));

		return (0);

	default:
		return (-EINVAL);
	}
}


/***************************************************************************
 * Hardware related stuff
 ***************************************************************************
 */

/***************************************************************************
 *
 * get RTC time:
 */

static unsigned long m41t00_get_rtc_time (void)
{
	unsigned char year, mon_cent, wday, mday, hour, min, sec;
	int yr;


	debugk ("m41t00.o:m41t00_get_rtc_time Entering \n");
	sec = rtc_rd (0x00);
	min = rtc_rd (0x01);
	hour = rtc_rd (0x02);
	mday = rtc_rd (0x04);
	wday = rtc_rd (0x03);
	mon_cent = rtc_rd (0x05);
	year = rtc_rd (0x06);

	debugk ("Get RTC year: %02x mon/cent: %02x mday: %02x wday: %02x "
		"hr: %02x min: %02x sec: %02x\n",
		year, mon_cent, mday, wday, hour, min, sec);

	yr = bcd2bin (year) + ((hour & 0x40) ? 2000 : 1900);
	debugk ("get rtc [dec] year=%d mon=%d day=%d hour=%d min=%d sec=%d\n",
		yr,
		bcd2bin (mon_cent & 0x1F),
		bcd2bin (mday & 0x3F),
		bcd2bin (hour & 0x3F),
		bcd2bin (min & 0x7F), bcd2bin (sec & 0x7F));

	return (mktime (yr,
			bcd2bin (mon_cent & 0x1F),
			bcd2bin (mday & 0x3F),
			bcd2bin (hour & 0x3F),
			bcd2bin (min & 0x7F), bcd2bin (sec & 0x7F)
		));
}

/***************************************************************************
 *
 * set RTC time:
 */
static int m41t00_set_rtc_time (unsigned long now)
{
	struct rtc_time tm;
	unsigned char century, year, mon, wday, mday, hour, min, sec;

	/* unsigned long flags; */
	/* extern time_t last_rtc_update; */
	/* extern rwlock_t xtime_lock; */
	/* write_lock_irqsave(&xtime_lock, flags); */
	to_tm (now, &tm);

	debugk ("Set RTC [dec] year=%d mon=%d day=%d hour=%d min=%d sec=%d\n",
		tm.tm_year, tm.tm_mon, tm.tm_mday,
		tm.tm_hour, tm.tm_min, tm.tm_sec);
	century = (tm.tm_year >= 2000) ? 0xC0 : 0x80;
	year = bin2bcd (tm.tm_year % 100);

	mon = bin2bcd (tm.tm_mon);
	wday = bin2bcd (tm.tm_wday);
	mday = bin2bcd (tm.tm_mday);
	hour = bin2bcd (tm.tm_hour) | century;;
	min = bin2bcd (tm.tm_min);
	sec = bin2bcd (tm.tm_sec);

	debugk ("Set RTC [bcd] year=%X mon=%X day=%X "
		"hour=%X min=%X sec=%X wday=%X\n",
		year, mon, mday, hour, min, sec, wday);

	rtc_wr (0x06, year);
	rtc_wr (0x05, mon);
	rtc_wr (0x03, wday);
	rtc_wr (0x04, mday);
	rtc_wr (0x02, hour);
	rtc_wr (0x01, min);
	rtc_wr (0x00, sec);

	/* write_unlock_irqrestore(&xtime_lock, flags); */
	return (0);
}

/***************************************************************************
 *
 * set RTC time:
 */
static void m41t00_reset_rtc (void)
{

	/*Set and clear ST bit */
	rtc_wr (0x00, rtc_rd (0x0) & 0x7F);
	/*Set and clear ST bit */
	rtc_wr (0x07, 0x0);
	/*Enable Century BIT */
	rtc_wr (0x02, rtc_rd (0x02) | 0x80);

}

/***************************************************************************
 * Module Declarations
 ***************************************************************************
 */
module_init (m41t00_rtc_init);

#ifdef MODULE
/*
 * Cleanup - unregister the appropriate file from /proc
 */
static void m41t00_rtc_cleanup (void)
{
	ppc_md.set_rtc_time = save_set_rtc_time;
	ppc_md.get_rtc_time = save_get_rtc_time;

	misc_deregister (&rtc_dev);

	debugk ("M41T00 Real-Time Clock Driver de-registered\n");
}

module_exit (m41t00_rtc_cleanup);
#endif /* MODULE */


/***************************************************************************
 *
 * Utility functions
 */

static unsigned int bcd2bin (unsigned int n)
{
	return ((((n >> 4) & 0x0F) * 10) + (n & 0x0F));
}

static unsigned int bin2bcd (unsigned int n)
{
	return (((n / 10) << 4) | (n % 10));
}

static void rtc_wr (unsigned char reg, unsigned char val)
{
	unsigned char buf[2];

	debugk ("m41t00.o: rtc_wr entering\n");
	debugk ("\treg: 0x%02X, val|: 0x%02X, clnt 0x%X\n", reg, val, clnt);
	buf[0] = reg;
	buf[1] = val;

	if (i2c_master_send (clnt, (char *) buf, 2) != 2) {
		printk (__FUNCTION__ "(): i2c transfer error\n");
	}

}

static unsigned char rtc_rd (unsigned char reg)
{
	unsigned char buf[1], addr[1];

	struct i2c_msg msgs[2] = {
	      {addr: clnt->addr, flags: 0, len: 1, buf:addr},
	      {addr: clnt->addr, flags: I2C_M_RD, len: 1, buf:buf}
	};

	/*  debugk("m41t00.o: rtc_rd entering \n"); */
	buf[0] = 0;
	addr[0] = reg;

/* 	if (down_interruptible (&mutex)) */
/*   		return (-ERESTARTSYS); */
	if (i2c_transfer (clnt->adapter, msgs, 2) != 2) {
		printk (__FUNCTION__ "(): i2c transfer error\n");
		return -1;
	}
/* 	up(&mutex); */
	/*  debugk("m41t00.o: rtc_rd exiting \n"); */

	return buf[0];
}
