/*
 * (C) Copyright 2004
 * DENX Software Engineering
 * Wolfgang Denk, wd@denx.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/i2c.h>
#include <linux/i2c-algo-bit.h>
#include <asm/8xx_immap.h>

#define SCL	0x1000	/* PA 3 */
#define SDA	0x2000	/* PA 2 */

#define PAR	immap->im_ioport.iop_papar
#define DIR	immap->im_ioport.iop_padir
#define DAT	immap->im_ioport.iop_padat

static void bit_nc650_setscl(void *data, int state)
{
	volatile immap_t *immap = (immap_t *)IMAP_ADDR;

	if (state)
		DAT |= SCL;
	else {
		DAT &=	~SCL;
	}
}

static void bit_nc650_setsda(void *data, int state)
{
	volatile immap_t *immap = (immap_t *)IMAP_ADDR;

	if (state)
		DIR &= ~SDA;
	else {
		DAT &= ~SDA;
		DIR |= SDA;
	}
}

static int bit_nc650_getscl(void *data)
{
	volatile immap_t *immap = (immap_t *)IMAP_ADDR;

	return (DAT & SCL) != 0;
}

static int bit_nc650_getsda(void *data)
{
	volatile immap_t *immap = (immap_t *)IMAP_ADDR;
	int res;
	res = (DAT & SDA) != 0;

	return res;
}

static void nc650_inc_use(struct i2c_adapter *adap)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}

static void
nc650_dec_use(struct i2c_adapter *adap)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}

static struct i2c_algo_bit_data nc650_algo_data = {
	NULL,
	bit_nc650_setsda,
	bit_nc650_setscl,
	bit_nc650_getsda,
	bit_nc650_getscl,
	80, 80, 100,		/* waits, timeout */
};

static struct i2c_adapter nc650_board_adapter = {
	name:              "NC650 I2C adapter",
	id:                0xfffc,
	algo:              NULL,
	algo_data:         &nc650_algo_data,
	inc_use:           nc650_inc_use,
	dec_use:           nc650_dec_use,
};

MODULE_DESCRIPTION("I2C adapter for NC650");
MODULE_LICENSE("GPL");

static __init int
i2c_nc650_init(void)
{
	volatile immap_t *immap = (immap_t *)IMAP_ADDR;

	PAR &= ~(SCL | SDA); DIR |= SCL;

	if (i2c_bit_add_bus(&nc650_board_adapter) < 0)
		return -ENODEV;

	return 0;
}

static __exit void
i2c_nc650_exit(void)
{
	i2c_bit_del_bus(&nc650_board_adapter);
}

module_init(i2c_nc650_init);
module_exit(i2c_nc650_exit);

