/*
 * drivers/mtd/maps/inka4x0.c
 *
 * MTD mapping for the INKA 4X0 board.
 *
 */
#include <linux/module.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>

#ifdef CONFIG_UBOOT
#include <asm/ppcboot.h>

extern unsigned char __res[];
bd_t *bd = (bd_t *)__res;

#define FLASH_ADDR  (bd->bi_flashstart)
#define FLASH_SIZE  (bd->bi_flashsize)
#else
#define FLASH_ADDR  CONFIG_PPC_5xxx_FLASH_ADDR
#define FLASH_SIZE  CONFIG_PPC_5xxx_FLASH_SIZE
#endif

#ifndef CONFIG_MTD_PARTITIONS
#error need to define CONFIG_MTD_PARTITIONS
#endif
static struct mtd_info *mymtd;

struct map_info inka4x0_map = {
	name:		"INKA4X0",
	bankwidth:	1,
};

static struct mtd_partition inka4x0_partition[] =
{
	{
		name:		"U-boot",
		size:		0x00040000,
		offset:		0
	},
	{
		name:		"Kernel",
                size:           0x0200000 - 0x00040000,
		offset:		MTDPART_OFS_APPEND
	},
};

#define NB_OF(x)	(sizeof(x)/sizeof(x[0]))

static int __init init_inka4x0 (void)
{
	printk (KERN_ALERT "INKA4X0 flash device: %lx at %lx\n", FLASH_SIZE,
		FLASH_ADDR);
	inka4x0_map.virt = inka4x0_map.phys = FLASH_ADDR;
	inka4x0_map.size = FLASH_SIZE;
	simple_map_init(&inka4x0_map);

	mymtd = do_map_probe ("cfi_probe", &inka4x0_map);
	if (!mymtd) {
		printk ("cfi_probe failed\n");
		return -ENXIO;
	}

	mymtd->owner = THIS_MODULE;
	printk (KERN_NOTICE "Using static partition definition\n");
	add_mtd_partitions (mymtd, inka4x0_partition, NB_OF (inka4x0_partition));

	return 0;
}


static void __exit cleanup_inka4x0 (void)
{
	if (mymtd) {
		del_mtd_device (mymtd);
		map_destroy (mymtd);
	}
}
module_init(init_inka4x0);
module_exit(cleanup_inka4x0);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("");
MODULE_DESCRIPTION("MTD map driver for INKA4X0 board");
