/*
 * Handle the NAND flash memory access routines 
 * on MATRIX VISION mvBLUE devices.
 *
 * $Id: mv_nand.c 1905 2005-06-10 11:21:42Z hg $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
  * Copyright(C) 2003-2005 MATRIX VISION GmbH <info@matrix-vision.de>
 *
 * 
 */

#include <linux/slab.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/partitions.h>
#include <asm/io.h>

static struct mtd_info *mvblue_mtd = NULL;
/*
 * Module stuff
 */
#if LINUX_VERSION_CODE < 0x20212 && defined(MODULE)
  #define mvblue_init init_module
  #define mvblue_cleanup cleanup_module
#endif

#ifdef CONFIG_MVBLUE_M
#define mvBLUE_IO_BASE      0xFF000008
#else
#define mvBLUE_IO_BASE      0xFF000020
#endif
#define mvBLUE_CTRL_BASE    0xFF000004

static int mvblue_io_base = mvBLUE_IO_BASE;
static int mvblue_ctrl_base = mvBLUE_CTRL_BASE;
static char *mvblue_nand_id = "mvBLUE-nand";

#define	__ROOT_FS_SIZE	(0x400000)

static struct mtd_partition partition_info[] = {
	{ .name = "root",
	  .offset = 0,
	  .size = __ROOT_FS_SIZE },
	{ .name = "user",
	  .offset = __ROOT_FS_SIZE,
	  .size = 0 }
};
#define NUM_PARTITIONS 2

static void mvblue_hwcontrol(struct mtd_info *mtd, int cmd){

    static unsigned char reg = 0x4;

    switch(cmd){

       case NAND_CTL_SETCLE: reg |=  0x01; break;
       case NAND_CTL_CLRCLE: reg &= ~0x01; break;

       case NAND_CTL_SETALE: reg |=  0x02; break;
       case NAND_CTL_CLRALE: reg &= ~0x02; break;

       case NAND_CTL_SETNCE: reg &= ~0x04; break;
       case NAND_CTL_CLRNCE: reg |=  0x04; break; 
    }
    *(volatile unsigned char *) (mvblue_ctrl_base) = reg;
}


static int mvblue_device_ready(struct mtd_info *mtd)
{
    return ( (*(volatile unsigned char *) (mvblue_ctrl_base)) & 0x80) ? 1 : 0;
}

#ifdef CONFIG_MVBLUE_M
unsigned char swaptab[256] = {
  0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0,
  0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0,
  0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8,
  0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8,
  0x04, 0x84, 0x44, 0xc4, 0x24, 0xa4, 0x64, 0xe4,
  0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4,
  0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec,
  0x1c, 0x9c, 0x5c, 0xdc, 0x3c, 0xbc, 0x7c, 0xfc,
  0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2,
  0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2,
  0x0a, 0x8a, 0x4a, 0xca, 0x2a, 0xaa, 0x6a, 0xea,
  0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa,
  0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6,
  0x16, 0x96, 0x56, 0xd6, 0x36, 0xb6, 0x76, 0xf6,
  0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee,
  0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe,
  0x01, 0x81, 0x41, 0xc1, 0x21, 0xa1, 0x61, 0xe1,
  0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1,
  0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9,
  0x19, 0x99, 0x59, 0xd9, 0x39, 0xb9, 0x79, 0xf9,
  0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5,
  0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5,
  0x0d, 0x8d, 0x4d, 0xcd, 0x2d, 0xad, 0x6d, 0xed,
  0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd,
  0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3,
  0x13, 0x93, 0x53, 0xd3, 0x33, 0xb3, 0x73, 0xf3,
  0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb,
  0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb,
  0x07, 0x87, 0x47, 0xc7, 0x27, 0xa7, 0x67, 0xe7,
  0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7,
  0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef,
  0x1f, 0x9f, 0x5f, 0xdf, 0x3f, 0xbf, 0x7f, 0xff,
};
    #define BS(b)     (swaptab[b])
#else
    #define BS(b)     (b)
#endif

static u_char mv_nand_read_byte(struct mtd_info *mtd)
{
    struct nand_chip *this = mtd->priv;
    return BS(readb(this->IO_ADDR_R));
}
static void mv_nand_write_byte(struct mtd_info *mtd, u_char byte)
{
    struct nand_chip *this = mtd->priv;
    writeb(BS(byte), this->IO_ADDR_W);
}
static void mv_nand_write_buf(struct mtd_info *mtd, const u_char *buf, int len)
{
    int i;
    for (i=0; i<len; i++)
        mv_nand_write_byte(mtd,buf[i]);
}
static void mv_nand_read_buf(struct mtd_info *mtd, u_char *buf, int len)
{
    int i;
    for (i=0; i<len; i++)
        buf[i] = mv_nand_read_byte(mtd);
}
static int mv_nand_verify_buf(struct mtd_info *mtd, const u_char *buf, int len)
{
    int i;
    for (i=0; i<len; i++)
        if (buf[i] != mv_nand_read_byte(mtd))
            return -EFAULT;
    return 0;
}





#ifdef CONFIG_MTD_CMDLINE_PARTS
extern int parse_mtd_partitions(struct mtd_info *master, const char **types,
				             struct mtd_partition **pparts, unsigned long origin);
static const char *part_probes[] = { "cmdlinepart", NULL };
#endif

/* Internal buffers. Page buffer and oob buffer for one block*/
static u_char data_buf[512 + 16];
static u_char oob_buf[16 * 32];

int __init mvblue_init (void)
{
	struct nand_chip *this;
	int err = 0;
	int mtd_parts_nb = 0;
	struct mtd_partition *mtd_parts = 0;

	/* Allocate memory for MTD device structure and private data */
	mvblue_mtd = kmalloc (sizeof(struct mtd_info) + sizeof (struct nand_chip),
				GFP_KERNEL);
	if (!mvblue_mtd) {
		printk ("Unable to allocate mvBLUE NAND MTD device structure.\n");
		return -ENOMEM;
	}

	this = (struct nand_chip *) (&mvblue_mtd[1]);

	memset((char *) mvblue_mtd, 0, sizeof(struct mtd_info));
	memset((char *) this, 0, sizeof(struct nand_chip));

	mvblue_mtd->priv = this;

    this->IO_ADDR_R = (void *) mvblue_io_base;
    this->IO_ADDR_W = (void *) mvblue_io_base;
	this->hwcontrol = mvblue_hwcontrol;
	this->chip_delay= 20;		
	this->dev_ready = mvblue_device_ready;
	this->read_byte = mv_nand_read_byte;
	this->write_byte= mv_nand_write_byte;
	this->read_buf  = mv_nand_read_buf;
	this->write_buf = mv_nand_write_buf;
	this->verify_buf= mv_nand_verify_buf;
	this->eccmode 	= NAND_ECC_SOFT;

    /* Set internal data buffer */
    this->data_buf = data_buf;
    this->oob_buf = oob_buf;

	if (nand_scan (mvblue_mtd, 1)) {
		kfree (mvblue_mtd);
		return -ENXIO;
	}
	/* unique identifier for NAND chip - replaces "NAND 32MiB 3,3V 8-bit" etc. */
	mvblue_mtd->name = mvblue_nand_id;

	/* Allocate memory for internal data buffer */
	this->data_buf = kmalloc (sizeof(u_char) * (mvblue_mtd->oobblock + mvblue_mtd->oobsize), GFP_KERNEL);
	if (!this->data_buf) {
		printk ("Unable to allocate NAND data buffer for mvBLUE device.\n");
		err = -ENOMEM;
		goto out_ior;
	}

	/* Register the partitions */
#ifdef CONFIG_MTD_CMDLINE_PARTS
	{
		mvblue_mtd->name = "mvBLUE-nand";
		mtd_parts_nb = parse_mtd_partitions(mvblue_mtd, part_probes, &mtd_parts, 0);
		
		if (mtd_parts_nb <= 0)
		{	/* default to mvBL-2xx/4xx partion map if parser fails */
			partition_info[1].size = mvblue_mtd->size - __ROOT_FS_SIZE;
			mtd_parts = partition_info;
			mtd_parts_nb = NUM_PARTITIONS;
		}
	}
#else 
	partition_info[1].size = mvblue_mtd->size - __ROOT_FS_SIZE;
	mtd_parts = partition_info;
	mtd_parts_nb = NUM_PARTITIONS;
#endif 
	add_mtd_partitions(mvblue_mtd, mtd_parts, mtd_parts_nb);
	goto out;

/* out_buf: */
	kfree (this->data_buf);    
out_ior:
out:
	return err;
}

module_init(mvblue_init);

#ifdef MODULE
static void __exit mvblue_cleanup (void)
{
	struct nand_chip *this = (struct nand_chip *) &mvblue_mtd[1];

    /* Unregister the device */
    del_mtd_partitions(mvblue_mtd);
	del_mtd_device (mvblue_mtd);

	kfree (this->data_buf);
	kfree (mvblue_mtd);
}
module_exit(mvblue_cleanup);
#endif

MODULE_LICENSE("GPL");
MODULE_AUTHOR("MATRIX VISION GmbH <info@matrix-vision.de>");
MODULE_DESCRIPTION("glue layer for NAND flash on mvBLUE device");

