
#ifndef __ISP1362GAPI_H 
#define __ISP1362GAPI_H

/*================================================
 * ISP1362 Device Controller interface
 */

#define	ISP1362_IO_PHYS		0xD0000000

#define	ISP1362_IRQ_HC		6
#define	ISP1362_IRQ_DC		8

/* #define	DEBUG */

#define	EP_DIR_IN		1
#define	EP_DIR_OUT		0

  /* Initialization commands
   */
#define	WR_DcEndpointConfiguration0_OUT		0x20
#define	WR_DcEndpointConfiguration0_IN		0x21
#define	WR_DcEndpointConfiguration		0x21 /* 1..14 */

#define	RD_DcEndpointConfiguration0_OUT		0x30
#define	RD_DcEndpointConfiguration0_IN		0x31
#define	RD_DcEndpointConfiguration		0x31 /* 1..14 */

#define	WR_DcAddress				0xB6
#define	RD_DcAddress				0xB7

#define	WR_DcMode				0xB8
#define	RD_DcMode				0xB9

#define	WR_DcHardwareConfiguration		0xBA
#define	RD_DcHardwareConfiguration		0xBB

#define	WR_DcInterruptEnable			0xC2
#define	RD_DcInterruptEnable			0xC3

#define	WR_DcDMAConfiguration			0xF0
#define	RD_DcDMAConfiguration			0xF1

#define	WR_DcDMACounter				0xF2
#define	RD_DcDMACounter				0xF3

#define	DO_ResetDevice				0xF6

  /* Data flow commands
   */
#define	DO_WriteEndpointBuffer0			0x01
#define	DO_WriteEndpointBuffer			0x01 /* 1..14 */

#define	DO_ReadEndpointBuffer0			0x10
#define	DO_ReadEndpointBuffer			0x11 /* 1..14 */

#define	DO_StallEndpoint0_OUT			0x40
#define	DO_StallEndpoint0_IN			0x41
#define	DO_StallEndpoint			0x41 /* 1..14 */

#define	RD_DcEndpointStatus0_OUT		0x50
#define	RD_DcEndpointStatus0_IN			0x51
#define	RD_DcEndpointStatus			0x51 /* 1..14 */

#define	DO_ValidateEndpointBuffer0		0x61
#define	DO_ValidateEndpointBuffer		0x61 /* 1..14 */

#define	DO_ClearEndpointBuffer0			0x70
#define	DO_ClearEndpointBuffer			0x71 /* 1..14 */

#define	DO_UnstallEndpoint0_OUT			0x80
#define	DO_UnstallEndpoint0_IN			0x81
#define	DO_UnstallEndpoint			0x81 /* 1..14 */

#define	RD_DcEndpointStatusImage0_OUT		0xD0
#define	RD_DcEndpointStatusImage0_IN		0xD1
#define	RD_DcEndpointStatusImage		0xD1 /* 1..14 */

#define	DO_AcknowledgeSetup			0xF4

  /* General commands
   */
#define	RD_DcErrorCode0_OUT			0xA0
#define	RD_DcErrorCode0_IN			0xA1
#define	RD_DcErrorCode				0xA1 /* 1..14 */

#define	WR_UnlockDevice				0xB0

#define	WR_DcScratch				0xB2
#define	RD_DcScratch				0xB3

#define	RD_DcFrameNumber			0xB4

#define	RD_DcChipID				0xB5

#define	RD_DcInterrupt				0xC0

  /* DcEndpointConfiguration register: bit allocation
   */
#define	EPCONF_FFOSZ_MASK			0x0F
#define	EPCONF_FFOSZ_NONISO_8			0x00
#define	EPCONF_FFOSZ_NONISO_16			0x01
#define	EPCONF_FFOSZ_NONISO_32			0x02
#define	EPCONF_FFOSZ_NONISO_64			0x03
#define	EPCONF_FFOSZ_ISO_16			0x00
#define	EPCONF_FFOSZ_ISO_32			0x01
#define	EPCONF_FFOSZ_ISO_48			0x02
#define	EPCONF_FFOSZ_ISO_64			0x03
#define	EPCONF_FFOSZ_ISO_96			0x04
#define	EPCONF_FFOSZ_ISO_128			0x05
#define	EPCONF_FFOSZ_ISO_160			0x06
#define	EPCONF_FFOSZ_ISO_192			0x07
#define	EPCONF_FFOSZ_ISO_256			0x08
#define	EPCONF_FFOSZ_ISO_320			0x09
#define	EPCONF_FFOSZ_ISO_384			0x0a
#define	EPCONF_FFOSZ_ISO_512			0x0b
#define	EPCONF_FFOSZ_ISO_640			0x0c
#define	EPCONF_FFOSZ_ISO_768			0x0d
#define	EPCONF_FFOSZ_ISO_896			0x0e
#define	EPCONF_FFOSZ_ISO_1023			0x0f

#define	EPCONF_FFOISO				0x10
#define	EPCONF_DBLBUF				0x20
#define	EPCONF_EPDIR_IN				0x40
#define	EPCONF_EPDIR_OUT			0x00
#define	EPCONF_FIFOEN				0x80

  /* DcMode register: bit allocation
   */
#define	MODE_SOFTCT				0x01
#define	MODE_DBGMOD				0x04
#define	MODE_INTENA				0x08
#define	MODE_GOSUSP				0x20

  /* DcHardwareConfiguration register: bit allocation
   */
#define	HWCONF_INTPOL				0x0001
#define	HWCONF_INTLVL				0x0002
#define	HWCONF_WKUPCS				0x0008
#define	HWCONF_DAKPOL				0x0020
#define	HWCONF_DRQPOL				0x0040
#define	HWCONF_DAKOLY				0x0080
#define	HWCONF_CKDIV_MASK			0x0F00
#define	HWCONF_CKDIV_12MHZ			0x0300
#define	HWCONF_CLKRUN				0x1000
#define	HWCONF_NOLAZY				0x2000
#define	HWCONF_EXTPUL				0x4000

  /* DcInterruptEnable/DcInterrupt registers: bit allocation
   */
#define	INT_RST					0x00000001
#define	INT_RESM				0x00000002
#define	INT_SUSP				0x00000004
#define	INT_EOT					0x00000008
#define	INT_SOF					0x00000010
#define	INT_PSOF				0x00000020
#define	INT_SP_EOT				0x00000040
#define	INT_EP0_OUT				0x00000100
#define	INT_EP0_IN				0x00000200
#define	INT_EP					0x00000200 /* 1..14 */
#define	INT_EP_ALL      			0x00FFFE00

  /* DcEndpointStatus register: bit allocation
   */
#define	EPSTAT_CPUBUF				0x02
#define	EPSTAT_SETUPT				0x04
#define	EPSTAT_OVERWRITE			0x08
#define	EPSTAT_DATA_PID				0x10
#define	EPSTAT_EPFULL0				0x20
#define	EPSTAT_EPFULL1				0x40
#define	EPSTAT_EPSTAL				0x80

  /* DcErrorCode register: bit allocation
   */
#define	EPERR_RTOK				0x01
#define	EPERR_ERR_MASK				0x1E
#define	EPERR_ERR_PID_ENC			0x02
#define	EPERR_ERR_PID_UNKNOWN			0x04
#define	EPERR_ERR_UNEXP_PACKET			0x06
#define	EPERR_ERR_TOKEN_CRC			0x08
#define	EPERR_ERR_DATA_CRC			0x0A
#define	EPERR_ERR_TIMEOUT			0x0C
#define	EPERR_ERR_BABBLE			0x0E
#define	EPERR_ERR_UNEXP_EOP			0x10
#define	EPERR_ERR_NAK				0x12
#define	EPERR_ERR_STALL				0x14
#define	EPERR_ERR_OVERFLOW			0x16
#define	EPERR_ERR_EMPTY_PACKET			0x18
#define	EPERR_ERR_BIT_STUFF			0x1A
#define	EPERR_ERR_SYNC				0x1C
#define	EPERR_ERR_WRONG_TOGGLE			0x1E
#define	EPERR_DATA01				0x40
#define	EPERR_UNREAD				0x80

#define ISP1362_UDC_NUM_ENDPOINTS 15

#define BULK_FIFO_SIZE 64
#define EP0_FIFO_SIZE 64

struct isp1362_udc;

struct isp1362_request {
	struct usb_request			req;
  	struct list_head			queue;
	unsigned                                zlp_sent;
};

struct isp1362_ep {
	struct usb_ep				ep;
	struct isp1362_udc			*dev;

	const struct usb_endpoint_descriptor	*desc;
	struct list_head			queue;	

	unsigned short		                fifo_size;
	u8					bEndpointAddress;
	u8					bmAttributes;
};

struct isp1362_udc {
  	struct usb_gadget		gadget;
	struct usb_gadget_driver	*driver;

	spinlock_t			lock;

	struct isp1362_ep		ep [ISP1362_UDC_NUM_ENDPOINTS];
	unsigned                        ep0dir_in;
};

#ifndef container_of
#define container_of    list_entry
#endif

/* Data extraction from usb_request_t fields */
static inline int request_target( __u8 b ) { return (int) ( b & 0x0F); }

static inline int windex_to_ep_num( __u16 w ) { return (int) ( w & 0x000F); }

static inline int ep_num(struct isp1362_ep* ep) 
{ 
	return ep->bEndpointAddress & ~USB_DIR_IN;
}

#endif
