/*
 * include/asm-ppc/mpc8220.h
 *
 * Prototypes, etc. for the Motorola mpc8220 embedded cpu chip
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * Based on mpc5xxx.h by Dale Farnsworth <dfarnsworth@mvista.com>
 *
 * 2003-2004 (c) MontaVista Software, Inc. This file is licensed under
 * the terms of the GNU General Public License version 2.  This program
 * is licensed "as is" without any warranty of any kind, whether express
 * or implied.
 */

#ifndef __ASM_MPC8220_H
#define __ASM_MPC8220_H

/*
 * The board-specific header files included above must define
 * _IO_BASE, _ISA_MEM_BASE, and PCI_DRAM_OFFSET if non-zero.  If not defined,
 * a default value of zero is supplied here.
 */
#ifndef _IO_BASE
#define _IO_BASE        0
#endif
#ifndef _ISA_MEM_BASE
#define _ISA_MEM_BASE   0
#endif
#ifndef PCI_DRAM_OFFSET
#define PCI_DRAM_OFFSET 0
#endif

#define	MPC8220_MBAR			0xf0000000

/* Memory Mapping */
#define MPC8220_CS			(MPC8220_MBAR + 0x0000)
/* SDRAM Controller */
#define MPC8220_SDRAMC			(MPC8220_MBAR + 0x00100)
/* G2 Core Bus Arbiter */
#define MPC8220_ARB			(MPC8220_MBAR + 0x00200)
/* System PLL & Power Management */
#define MPC8220_SYSPLL			(MPC8220_MBAR + 0x00300)
/* Video PLL */
#define MPC8220_VIDPLL			(MPC8220_MBAR + 0x00400)
/* FlexBus Controller */
#define MPC8220_FLEXBUS			(MPC8220_MBAR + 0x00500)
/* Pin Multiplexing */
#define MPC8220_PORTCFG			(MPC8220_MBAR + 0x00600)
/* Interrupt Controller */
#define MPC8220_INTCTL			(MPC8220_MBAR + 0x00700)
/* General Purpose Timer */
#define MPC8220_GPT			(MPC8220_MBAR + 0x00800)
/* Slice Timer */
#define MPC8220_SLT			(MPC8220_MBAR + 0x00900)
/* GPIO */
#define MPC8220_GPIO			(MPC8220_MBAR + 0x00a00)
/* PCI */
#define MPC8220_PCI			(MPC8220_MBAR + 0x00b00)
/* PCI Arbiter */
#define MPC8220_PCI_ARB			(MPC8220_MBAR + 0x00c00)
/* External DMA Module 1 */
#define MPC8220_EXTDMA1			(MPC8220_MBAR + 0x00d00)
/* External DMA Module 2 */
#define MPC8220_EXTDMA2			(MPC8220_MBAR + 0x00e00)
/* USB Host Controller */
#define MPC8220_USB_HOST		(MPC8220_MBAR + 0x01000)
/* Comms Sub-System Timer */
#define MPC8220_COM_TIMER		(MPC8220_MBAR + 0x07f00)
/* Multi-Channel DMA */
#define MPC8220_SDMA			(MPC8220_MBAR + 0x08000)
/* USB Device */
#define MPC8220_USB_DEV			(MPC8220_MBAR + 0x08200)
/* Multi-Channel DMA PCI */
#define MPC8220_PCI_DMA			(MPC8220_MBAR + 0x08400)
/* Parallel Port */
#define MPC8220_1284			(MPC8220_MBAR + 0x08500)
/* PEV / DEV video  */
#define MPC8220_PEV_DEV			(MPC8220_MBAR + 0x08600)
/* Programmable Serial Controller */
#define MPC8220_PSC			(MPC8220_MBAR + 0x08800)
/* I2C */
#define MPC8220_I2C			(MPC8220_MBAR + 0x08f00)
/* Fast Ethernet Controller 1 */
#define MPC8220_FEC1			(MPC8220_MBAR + 0x09000)
/* Fast Ethernet Controller 2 */
#define MPC8220_FEC2			(MPC8220_MBAR + 0x09800)
/* JBIG configuration */
#define MPC8220_JBIG			(MPC8220_MBAR + 0x0d000)
/* PF300 Image Coprocessor */
#define MPC8220_PFM			(MPC8220_MBAR + 0x10000)
/* On-chip SRAM configuration */
#define MPC8220_SRAMCFG			(MPC8220_MBAR + 0x1ff00)
/* 32kB on-chip SRAM */
#define MPC8220_SRAM_SIZE		(32*1024)
#define MPC8220_SRAM_START		(MPC8220_MBAR + 0x20000)
#define MPC8220_SRAM_STOP		(MPC8220_SRAM_START + MPC8220_SRAM_SIZE - 1)

/* Memory Mapping Registers */
#define MPC_SDRAMDS			(MPC8220_CS + 0x04) /* SDRAM Drive Strength */
#define MPC_CS0CFG			(MPC8220_CS + 0x20) /* SDRAM Chip Select 0 Config */
#define MPC_CS1CFG			(MPC8220_CS + 0x24) /* SDRAM Chip Select 1 Config */
#define MPC_CS2CFG			(MPC8220_CS + 0x28) /* SDRAM Chip Select 2 Config */
#define MPC_CS3CFG			(MPC8220_CS + 0x2c) /* SDRAM Chip Select 3 Config */
#define MPC_CS4CFG			(MPC8220_CS + 0x30) /* SDRAM Chip Select 4 Config */
#define MPC_CS5CFG			(MPC8220_CS + 0x34) /* SDRAM Chip Select 5 Config */

#define MPC_XLB_CONFIG			(MPC8220_ARB + 0x40)  /* Arbiter configuration */
#define MPC_XLB_STATUS			(MPC8220_ARB + 0x48)
#define MPC_XLB_ADRCAP			(MPC8220_ARB + 0x50)
#define MPC_XLB_SIGCAP			(MPC8220_ARB + 0x54)
#define MPC_XLB_ADRTO			(MPC8220_ARB + 0x58)
#define MPC_XLB_DATTO			(MPC8220_ARB + 0x5c)
#define MPC_XLB_BUSTO			(MPC8220_ARB + 0x60)
#define MPC_XLB_PRIEN			(MPC8220_ARB + 0x064) /* Master Priority Enable */
#define MPC_XLB_PRI			(MPC8220_ARB + 0x068) /* Master Priority */

/* Pin Multiplexing registers */
#define MPC8220_PCFG0			(MPC8220_PORTCFG + 0x00) /* port cfg reg 0 */
#define MPC8220_PCFG1			(MPC8220_PORTCFG + 0x04) /* port cfg reg 1 */
#define MPC8220_PCFG2			(MPC8220_PORTCFG + 0x08) /* port cfg reg 2 */
#define MPC8220_PCFG3			(MPC8220_PORTCFG + 0x0c) /* port cfg reg 3 */

	/* System PLL Control */
#define MPC8220_PLL_SPCR		(MPC8220_SYSPLL + 0x000)

/* PEV / DEV video registers */
#define MPC8220_PEV_DEV_VCR		(MPC8220_PEV_DEV + 0x04) /* control */

/* USB device registers */
#define MPC8220_USB_FRAME					(MPC8220_USB_DEV + 0x000)
#define MPC8220_USB_SPEC					(MPC8220_USB_DEV + 0x004)
#define MPC8220_USB_STAT					(MPC8220_USB_DEV + 0x008)
#define MPC8220_USB_CTRL					(MPC8220_USB_DEV + 0x00C)
#define MPC8220_USB_DADR					(MPC8220_USB_DEV + 0x010)
#define MPC8220_USB_DDAT					(MPC8220_USB_DEV + 0x014)
#define MPC8220_USB_INTR					(MPC8220_USB_DEV + 0x018)
#define MPC8220_USB_MASK					(MPC8220_USB_DEV + 0x01C)
#define MPC8220_USB_MCTL					(MPC8220_USB_DEV + 0x020)
#define MPC8220_USB_EP_STAT(a)				(MPC8220_USB_DEV + 0x30 + (a*0x30))
#define MPC8220_USB_EP_INTR(a)				(MPC8220_USB_DEV + 0x34 + (a*0x30))
#define MPC8220_USB_EP_MASK(a)				(MPC8220_USB_DEV + 0x38 + (a*0x30))
#define MPC8220_USB_EP_FDAT_8(a)			(MPC8220_USB_DEV + 0x3C + (a*0x30))
#define MPC8220_USB_EP_FDAT_16(a)			(MPC8220_USB_DEV + 0x3C + (a*0x30))
#define MPC8220_USB_EP_FDAT(a)				(MPC8220_USB_DEV + 0x3C + (a*0x30))
#define MPC8220_USB_EP_FDAT_ADDR(a)			(MPC8220_USB_DEV + 0x3C + (a*0x30))
#define MPC8220_USB_EP_FSTAT(a)				(MPC8220_USB_DEV + 0x40 + (a*0x30))
#define MPC8220_USB_EP_FCTRL(a)				(MPC8220_USB_DEV + 0x44 + (a*0x30))
#define MPC8220_USB_EP_LRFP(a)				(MPC8220_USB_DEV + 0x48 + (a*0x30))
#define MPC8220_USB_EP_LWFP(a)				(MPC8220_USB_DEV + 0x4C + (a*0x30))
#define MPC8220_USB_EP_FALRM(a)				(MPC8220_USB_DEV + 0x50 + (a*0x30))
#define MPC8220_USB_EP_FRDP(a)				(MPC8220_USB_DEV + 0x54 + (a*0x30))
#define MPC8220_USB_EP_FWRP(a)				(MPC8220_USB_DEV + 0x58 + (a*0x30))
#define MPC8220_USB_EP0_STAT				(MPC8220_USB_DEV + 0x030)
#define MPC8220_USB_EP0_INTR				(MPC8220_USB_DEV + 0x034)
#define MPC8220_USB_EP0_MASK				(MPC8220_USB_DEV + 0x038)
#define MPC8220_USB_EP0_FDAT				(MPC8220_USB_DEV + 0x03C)
#define MPC8220_USB_EP0_FSTAT				(MPC8220_USB_DEV + 0x040)
#define MPC8220_USB_EP0_FCTRL				(MPC8220_USB_DEV + 0x044)
#define MPC8220_USB_EP0_LRFP				(MPC8220_USB_DEV + 0x048)
#define MPC8220_USB_EP0_LWFP				(MPC8220_USB_DEV + 0x04C)
#define MPC8220_USB_EP0_FALRM				(MPC8220_USB_DEV + 0x050)
#define MPC8220_USB_EP0_FRDP				(MPC8220_USB_DEV + 0x054)
#define MPC8220_USB_EP0_FWRP				(MPC8220_USB_DEV + 0x058)
#define MPC8220_USB_EP1_STAT  				(MPC8220_USB_DEV + 0x060)
#define MPC8220_USB_EP1_INTR				(MPC8220_USB_DEV + 0x064)
#define MPC8220_USB_EP1_MASK				(MPC8220_USB_DEV + 0x068)
#define MPC8220_USB_EP1_FDAT				(MPC8220_USB_DEV + 0x06C)
#define MPC8220_USB_EP1_FSTAT				(MPC8220_USB_DEV + 0x070)
#define MPC8220_USB_EP1_FCTRL				(MPC8220_USB_DEV + 0x074)
#define MPC8220_USB_EP1_LRFP				(MPC8220_USB_DEV + 0x078)
#define MPC8220_USB_EP1_LWFP				(MPC8220_USB_DEV + 0x07C)
#define MPC8220_USB_EP1_FALRM				(MPC8220_USB_DEV + 0x080)
#define MPC8220_USB_EP1_FRDP				(MPC8220_USB_DEV + 0x084)
#define MPC8220_USB_EP1_FWRP				(MPC8220_USB_DEV + 0x088)
#define MPC8220_USB_EP2_STAT				(MPC8220_USB_DEV + 0x090)
#define MPC8220_USB_EP2_INTR				(MPC8220_USB_DEV + 0x094)
#define MPC8220_USB_EP2_MASK				(MPC8220_USB_DEV + 0x098)
#define MPC8220_USB_EP2_FDAT				(MPC8220_USB_DEV + 0x09C)
#define MPC8220_USB_EP2_FSTAT				(MPC8220_USB_DEV + 0x0A0)
#define MPC8220_USB_EP2_FCTRL 				(MPC8220_USB_DEV + 0x0A4)
#define MPC8220_USB_EP2_LRFP				(MPC8220_USB_DEV + 0x0A8)
#define MPC8220_USB_EP2_LWFP				(MPC8220_USB_DEV + 0x0AC)
#define MPC8220_USB_EP2_FALRM				(MPC8220_USB_DEV + 0x0B0)
#define MPC8220_USB_EP2_FRDP				(MPC8220_USB_DEV + 0x0B4)
#define MPC8220_USB_EP2_FWRP				(MPC8220_USB_DEV + 0x0B8)

/* IEEE-1284 device registers */
#define MPC8220_1284MODESUPT				(MPC8220_1284 + 0x002)
#define MPC8220_1284PCON				(MPC8220_1284 + 0x003)
#define MPC8220_1284INTCTRL				(MPC8220_1284 + 0x004)
#define MPC8220_1284BUFSTAT				(MPC8220_1284 + 0x005)
#define MPC8220_1284ADDR				(MPC8220_1284 + 0x006)
#define MPC8220_1284OUTCTRL				(MPC8220_1284 + 0x007)
#define MPC8220_1284OUTSTAT_DATA			(MPC8220_1284 + 0x008)
#define MPC8220_1284INSTAT				(MPC8220_1284 + 0x009)
#define MPC8220_1284DATA				(MPC8220_1284 + 0x00a)
#define MPC8220_1284REV					(MPC8220_1284 + 0x00b)
#define MPC8220_1284INTSTAT				(MPC8220_1284 + 0x00c)
#define MPC8220_1284MODE				(MPC8220_1284 + 0x00d)
#define MPC8220_1284COMM				(MPC8220_1284 + 0x00e)
#define MPC8220_1284VECTOR				(MPC8220_1284 + 0x00f)
#define MPC8220_1284FIFODATA				(MPC8220_1284 + 0x010)
#define MPC8220_1284FIFODATA8				(MPC8220_1284 + 0x013)
#define MPC8220_1284FIFOSTAT				(MPC8220_1284 + 0x014)
#define MPC8220_1284FIFOCTRL				(MPC8220_1284 + 0x018)
#define MPC8220_1284FIFOALARM				(MPC8220_1284 + 0x01c)
#define MPC8220_1284FIFORDPTR				(MPC8220_1284 + 0x020)
#define MPC8220_1284FIFOWRPTR				(MPC8220_1284 + 0x024)

/* Programmable Serial Controller (PSC) status register bits */
#define MPC8220_PSC_SR_ERR	0x0040
#define MPC8220_PSC_SR_CDE	0x0080
#define MPC8220_PSC_SR_RXRDY	0x0100
#define MPC8220_PSC_SR_RXFULL	0x0200
#define MPC8220_PSC_SR_TXRDY	0x0400
#define MPC8220_PSC_SR_TXEMP	0x0800
#define MPC8220_PSC_SR_OE	0x1000
#define MPC8220_PSC_SR_PE	0x2000
#define MPC8220_PSC_SR_FE	0x4000
#define MPC8220_PSC_SR_RB	0x8000

/* PSC Command values */
#define MPC8220_PSC_RX_ENABLE		0x0001
#define MPC8220_PSC_RX_DISABLE		0x0002
#define MPC8220_PSC_TX_ENABLE		0x0004
#define MPC8220_PSC_TX_DISABLE		0x0008
#define MPC8220_PSC_SEL_MODE_REG_1	0x0010
#define MPC8220_PSC_RST_RX		0x0020
#define MPC8220_PSC_RST_TX		0x0030
#define MPC8220_PSC_RST_ERR_STAT	0x0040
#define MPC8220_PSC_RST_BRK_CHG_INT	0x0050
#define MPC8220_PSC_START_BRK		0x0060
#define MPC8220_PSC_STOP_BRK		0x0070

/* PSC Rx FIFO status bits */
#define MPC8220_PSC_RX_FIFO_ERR		0x0040
#define MPC8220_PSC_RX_FIFO_UF		0x0020
#define MPC8220_PSC_RX_FIFO_OF		0x0010
#define MPC8220_PSC_RX_FIFO_FR		0x0008
#define MPC8220_PSC_RX_FIFO_FULL	0x0004
#define MPC8220_PSC_RX_FIFO_ALARM	0x0002
#define MPC8220_PSC_RX_FIFO_EMPTY	0x0001

/* PSC interrupt mask bits */
#define MPC8220_PSC_IMR_ERR		0x0040
#define MPC8220_PSC_IMR_TXRDY		0x0100
#define MPC8220_PSC_IMR_RXRDY		0x0200
#define MPC8220_PSC_IMR_DB		0x0400
#define MPC8220_PSC_IMR_IPC		0x8000

/* PSC input port change bit */
#define MPC8220_PSC_CTS			0x01
#define MPC8220_PSC_DCD			0x02

/* PSC mode reg 1 fields */
#define MPC8220_PSC_MODE_5_BITS			0x00
#define MPC8220_PSC_MODE_6_BITS			0x01
#define MPC8220_PSC_MODE_7_BITS			0x02
#define MPC8220_PSC_MODE_8_BITS			0x03
#define MPC8220_PSC_MODE_PAREVEN		0x00
#define MPC8220_PSC_MODE_PARODD			0x04
#define MPC8220_PSC_MODE_PARFORCE		0x08
#define MPC8220_PSC_MODE_PARNONE		0x10
#define MPC8220_PSC_MODE_ERR			0x20
#define MPC8220_PSC_MODE_FFULL			0x40
#define MPC8220_PSC_MODE_RXRTS			0x80

/* PSC mode reg 2 fields */
#define MPC8220_PSC_MODE_ONE_STOP_5_BITS	0x00
#define MPC8220_PSC_MODE_ONE_STOP		0x07
#define MPC8220_PSC_MODE_TWO_STOP		0x0f

#define MPC8220_PSC_RFNUM_MASK	0x01ff

/* Interrupt sources */
#define MPC8220_CRIT_IRQ_NUM	4
#define MPC8220_MAIN_IRQ_NUM	17
#define MPC8220_SDMA_IRQ_NUM	17
#define MPC8220_PERP_IRQ_NUM	22

#define MPC8220_CRIT_IRQ_BASE	0
#define MPC8220_MAIN_IRQ_BASE	(MPC8220_CRIT_IRQ_BASE + MPC8220_CRIT_IRQ_NUM)
#define MPC8220_SDMA_IRQ_BASE	(MPC8220_MAIN_IRQ_BASE + MPC8220_MAIN_IRQ_NUM)
#define MPC8220_PERP_IRQ_BASE	(MPC8220_SDMA_IRQ_BASE + MPC8220_SDMA_IRQ_NUM)

#define MPC8220_IRQ0			(MPC8220_CRIT_IRQ_BASE + 0)
#define MPC8220_SLICE_TIMER_1_IRQ	(MPC8220_CRIT_IRQ_BASE + 1)
#define MPC8220_HI_INT_IRQ		(MPC8220_CRIT_IRQ_BASE + 2)

#define MPC8220_IRQ1		(MPC8220_MAIN_IRQ_BASE + 1)
#define MPC8220_GPIO_IRQ	(MPC8220_MAIN_IRQ_BASE + 2)
#define MPC8220_PSC_IRQ		(MPC8220_PERP_IRQ_BASE + 14)

#define MPC8220_FEC1_IRQ		(MPC8220_PERP_IRQ_BASE + 9)
#define MPC8220_FEC2_IRQ		(MPC8220_PERP_IRQ_BASE + 10)

#define MPC8220_USB_HOST_IRQ		(MPC8220_PERP_IRQ_BASE + 20)
#define MPC8220_USB_DEVICE_IRQ		(MPC8220_PERP_IRQ_BASE + 3)
#define MPC8220_1284_IRQ		(MPC8220_PERP_IRQ_BASE + 13)

#ifndef __ASSEMBLY__

extern u8 mpc8220_fec1_mac_addr[6];
extern u8 mpc8220_fec2_mac_addr[6];

/*
 * PCI Header Registers
 * - may have to access all as 32 bits;
 * - the order is screwed up since we are big endian?
 */
struct mpc8220_xipci {
	u32	dev_ven_id;		/* 0xb00 - device/vendor ID */
	u32	stat_cmd_reg;		/* 0xb04 - status command register */
	u32	class_code_rev_id;	/* 0xb08 - class code / revision ID */
	u32	bist_htyp_lat_cshl;	/* 0xb0c - BIST/HeaderType/Latency/cache line */
	u32	base0;			/* 0xb10 - base address 0 */
	u32	base1;			/* 0xb14 - base address 1 */
	u32	reserved1[4];   	/* 0xb18->0xd27 - base address 2 - 5 */
	u32	cis;			/* 0xb28 - cardBus CIS pointer */
	u32	sub_sys_ven_id;		/* 0xb2c - sub system ID/ subsystem vendor ID */
	u32	reserved2;		/* 0xb30 - expansion ROM base address */
	u32	reserved3;		/* 0xb00 - reserved */
	u32	reserved4;		/* 0xb00 - reserved */
	u32	mlat_mgnt_ipl;		/* 0xb3c - MaxLat/MinGnt/ int pin/int line */
	u32	reserved5[8];
	/* MPC8220 specific - not accessible in PCI header space externally */
	u32	glb_stat_ctl;		/* 0xb60 - Global Status Control */
	u32	target_bar0;		/* 0xb64 - Target Base Address 0 */
	u32	target_bar1;		/* 0xb68 - Target Base Address 1 */
	u32	target_ctrl;		/* 0xb6c - Target Control */
	u32	init_win0;		/* 0xb70 - Initiator Window 0 Base/Translation */
	u32	init_win1;		/* 0xb74 - Initiator Window 1 Base/Translation */
	u32	init_win2;		/* 0xb78 - Initiator Window 2 Base/Translation */
	u32	reserved6;		/* 0xb7c - reserved  */
	u32	init_win_cfg; 		/* 0xb80 */
	u32	init_ctrl;   		/* 0xb84 */
	u32	init_stat;   		/* 0xb88 */
	u32	reserved7[27];
	u32	cfg_adr; 		/* 0xbf8 */
	u32	reserved8;
};
struct mpc8220_psc {
	u8	mode;				/* PSC + 0x00 */
	u8	reserved0[3];
	union {						/* PSC + 0x04 */
		u16	status;
		u16	clock_select;
	} sr_csr;
#define mpc8220_psc_status		sr_csr.status
#define mpc8220_psc_clock_select	sr_csr.clock_select
	u16	reserved1;
	u8	command;			/* PSC + 0x08 */
	u8	reserved2[3];
	union {						/* PSC + 0x0c */
		u8	buffer_8;
		u16	buffer_16;
		u32	buffer_32;
	} buffer;
#define mpc8220_psc_buffer_8	buffer.buffer_8
#define mpc8220_psc_buffer_16	buffer.buffer_16
#define mpc8220_psc_buffer_32	buffer.buffer_32
	union {						/* PSC + 0x10 */
		u8	ipcr;
		u8	acr;
	} ipcr_acr;
#define mpc8220_psc_ipcr	ipcr_acr.ipcr
#define mpc8220_psc_acr		ipcr_acr.acr
	u8	reserved3[3];
	union {						/* PSC + 0x14 */
		u16	isr;
		u16	imr;
	} isr_imr;
#define mpc8220_psc_isr		isr_imr.isr
#define mpc8220_psc_imr		isr_imr.imr
	u16	reserved4;
	u8	ctur;		/* PSC + 0x18 */
	u8	reserved5[3];
	u8	ctlr;		/* PSC + 0x1c */
	u8	reserved6[19];
	u8	ivr;		/* PSC + 0x30 */
	u8	reserved8[3];
	u8	ip;		/* PSC + 0x34 */
	u8	reserved9[3];
	u8	op1;		/* PSC + 0x38 */
	u8	reserved10[3];
	u8	op0;		/* PSC + 0x3c */
	u8	reserved11[3];
	u8	sicr;		/* PSC + 0x40 */
	u8	reserved12[3];
	u8	ircr1;		/* PSC + 0x44 */
	u8	reserved13[3];
	u8	ircr2;		/* PSC + 0x48 */
	u8	reserved14[3];
	u8	irsdr;		/* PSC + 0x4c */
	u8	reserved15[3];
	u8	irmdr;		/* PSC + 0x50 */
	u8	reserved16[3];
	u8	irfdr;		/* PSC + 0x54 */
	u8	reserved17[3];
	u16	rfnum;		/* PSC + 0x58 */
	u16	reserved18;
	u16	tfnum;		/* PSC + 0x5c */
	u16	reserved19;
	u32	rfdata;		/* PSC + 0x60 */
	u16	rfstat;		/* PSC + 0x64 */
	u16	reserved20;
	u8	rfcntl;		/* PSC + 0x68 */
	u8	reserved21[5];
	u16	rfalarm;	/* PSC + 0x6e */
	u16	reserved22;
	u16	rfrptr;		/* PSC + 0x72 */
	u16	reserved23;
	u16	rfwptr;		/* PSC + 0x76 */
	u16	reserved24;
	u16	rflrfptr;	/* PSC + 0x7a */
	u16	reserved25;
	u16	rflwfptr;	/* PSC + 0x7e */
	u32	tfdata;		/* PSC + 0x80 */
	u16	tfstat;		/* PSC + 0x84 */
	u16	reserved26;
	u8	tfcntl;		/* PSC + 0x88 */
	u8	reserved27[5];
	u16	tfalarm;	/* PSC + 0x8e */
	u16	reserved28;
	u16	tfrptr;		/* PSC + 0x92 */
	u16	reserved29;
	u16	tfwptr;		/* PSC + 0x96 */
	u16	reserved30;
	u16	tflrfptr;	/* PSC + 0x9a */
	u16	reserved31;
	u16	tflwfptr;	/* PSC + 0x9e */
};
struct mpc8220_intr {
	u32	per_mask;	/* INTR + 0x00 */
	u32	per_pri1;	/* INTR + 0x04 */
	u32	per_pri2;	/* INTR + 0x08 */
	u32	per_pri3;	/* INTR + 0x0c */
	u32	ctrl;		/* INTR + 0x10 */
	u32	main_mask;	/* INTR + 0x14 */
	u32	main_pri1;	/* INTR + 0x18 */
	u32	main_pri2;	/* INTR + 0x1c */
	u32	reserved1;	/* INTR + 0x20 */
	u32	enc_status;	/* INTR + 0x24 */
	u32	crit_status;	/* INTR + 0x28 */
	u32	main_status;	/* INTR + 0x2c */
	u32	per_status;	/* INTR + 0x30 */
	u32	reserved2;	/* INTR + 0x34 */
	u32	per_error;	/* INTR + 0x38 */
};

/* Multi-Channel DMA */
struct mpc8220_sdma {
	u32 taskBar;		/* SDMA + 0x00 */
	u32 currentPointer;	/* SDMA + 0x04 */
	u32 endPointer;	/* SDMA + 0x08 */
	u32 variablePointer;	/* SDMA + 0x0c */

	u8 IntVect1;		/* SDMA + 0x10 */
	u8 IntVect2;		/* SDMA + 0x11 */
	u16 PtdCntrl;		/* SDMA + 0x12 */

	u32 IntPend;		/* SDMA + 0x14 */
	u32 IntMask;		/* SDMA + 0x18 */

	u16 tcr_0;		/* SDMA + 0x1c */
	u16 tcr_1;		/* SDMA + 0x1e */
	u16 tcr_2;		/* SDMA + 0x20 */
	u16 tcr_3;		/* SDMA + 0x22 */
	u16 tcr_4;		/* SDMA + 0x24 */
	u16 tcr_5;		/* SDMA + 0x26 */
	u16 tcr_6;		/* SDMA + 0x28 */
	u16 tcr_7;		/* SDMA + 0x2a */
	u16 tcr_8;		/* SDMA + 0x2c */
	u16 tcr_9;		/* SDMA + 0x2e */
	u16 tcr_a;		/* SDMA + 0x30 */
	u16 tcr_b;		/* SDMA + 0x32 */
	u16 tcr_c;		/* SDMA + 0x34 */
	u16 tcr_d;		/* SDMA + 0x36 */
	u16 tcr_e;		/* SDMA + 0x38 */
	u16 tcr_f;		/* SDMA + 0x3a */

	u8 IPR0;		/* SDMA + 0x3c */
	u8 IPR1;		/* SDMA + 0x3d */
	u8 IPR2;		/* SDMA + 0x3e */
	u8 IPR3;		/* SDMA + 0x3f */
	u8 IPR4;		/* SDMA + 0x40 */
	u8 IPR5;		/* SDMA + 0x41 */
	u8 IPR6;		/* SDMA + 0x42 */
	u8 IPR7;		/* SDMA + 0x43 */
	u8 IPR8;		/* SDMA + 0x44 */
	u8 IPR9;		/* SDMA + 0x45 */
	u8 IPR10;		/* SDMA + 0x46 */
	u8 IPR11;		/* SDMA + 0x47 */
	u8 IPR12;		/* SDMA + 0x48 */
	u8 IPR13;		/* SDMA + 0x49 */
	u8 IPR14;		/* SDMA + 0x4a */
	u8 IPR15;		/* SDMA + 0x4b */
	u8 IPR16;		/* SDMA + 0x4c */
	u8 IPR17;		/* SDMA + 0x4d */
	u8 IPR18;		/* SDMA + 0x4e */
	u8 IPR19;		/* SDMA + 0x4f */
	u8 IPR20;		/* SDMA + 0x50 */
	u8 IPR21;		/* SDMA + 0x51 */
	u8 IPR22;		/* SDMA + 0x52 */
	u8 IPR23;		/* SDMA + 0x53 */
	u8 IPR24;		/* SDMA + 0x54 */
	u8 IPR25;		/* SDMA + 0x55 */
	u8 IPR26;		/* SDMA + 0x56 */
	u8 IPR27;		/* SDMA + 0x57 */
	u8 IPR28;		/* SDMA + 0x58 */
	u8 IPR29;		/* SDMA + 0x59 */
	u8 IPR30;		/* SDMA + 0x5a */
	u8 IPR31;		/* SDMA + 0x5b */

	u32 res1;		/* SDMA + 0x5c */
	u32 res2;		/* SDMA + 0x60 */
	u32 res3;		/* SDMA + 0x64 */
	u32 MDEDebug;		/* SDMA + 0x68 */
	u32 ADSDebug;		/* SDMA + 0x6c */
	u32 Value1;		/* SDMA + 0x70 */
	u32 Value2;		/* SDMA + 0x74 */
	u32 Control;		/* SDMA + 0x78 */
	u32 Status;		/* SDMA + 0x7c */
	u32 EU00;		/* SDMA + 0x80 */
	u32 EU01;		/* SDMA + 0x84 */
	u32 EU02;		/* SDMA + 0x88 */
	u32 EU03;		/* SDMA + 0x8c */
	u32 EU04;		/* SDMA + 0x90 */
	u32 EU05;		/* SDMA + 0x94 */
	u32 EU06;		/* SDMA + 0x98 */
	u32 EU07;		/* SDMA + 0x9c */
	u32 EU10;		/* SDMA + 0xa0 */
	u32 EU11;		/* SDMA + 0xa4 */
	u32 EU12;		/* SDMA + 0xa8 */
	u32 EU13;		/* SDMA + 0xac */
	u32 EU14;		/* SDMA + 0xb0 */
	u32 EU15;		/* SDMA + 0xb4 */
	u32 EU16;		/* SDMA + 0xb8 */
	u32 EU17;		/* SDMA + 0xbc */
	u32 EU20;		/* SDMA + 0xc0 */
	u32 EU21;		/* SDMA + 0xc4 */
	u32 EU22;		/* SDMA + 0xc8 */
	u32 EU23;		/* SDMA + 0xcc */
	u32 EU24;		/* SDMA + 0xd0 */
	u32 EU25;		/* SDMA + 0xd4 */
	u32 EU26;		/* SDMA + 0xd8 */
	u32 EU27;		/* SDMA + 0xdc */
	u32 EU30;		/* SDMA + 0xe0 */
	u32 EU31;		/* SDMA + 0xe4 */
	u32 EU32;		/* SDMA + 0xe8 */
	u32 EU33;		/* SDMA + 0xec */
	u32 EU34;		/* SDMA + 0xf0 */
	u32 EU35;		/* SDMA + 0xf4 */
	u32 EU36;		/* SDMA + 0xf8 */
	u32 EU37;		/* SDMA + 0xfc */
};

void mpc8220_init_irq(void);
int mpc8220_get_irq(struct pt_regs *regs);

#endif /* __ASSEMBLY__ */

#endif /* __ASMPPC_MPC8220_H */
