/*
 * nc650_cpld.c - Driver for CPLD functionality on NC650
 *
 * Copyright (C) 2005 Detlev Zundel <dzu@denx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef _nc650_cpld_h
#define _nc650_cpld_h

typedef struct {
	unsigned port;
	int mode;
} nc650_cpld_param_t;

/* Constants for the mode fields */
#define NC650_CPLD_MODE_INVALID 0
#define NC650_CPLD_MODE_RS232   1
#define NC650_CPLD_MODE_RS485   2
#define NC650_CPLD_MODE_OC      3

#define	NC650_CPLD_IOCTL_BASE	'C'

/* Defined operations */
#define NC650_CPLD_COMM_MODE_GET	_IOWR(NC650_CPLD_IOCTL_BASE, 0, unsigned)
#define NC650_CPLD_COMM_MODE_SET	_IOW (NC650_CPLD_IOCTL_BASE, 1, nc650_cpld_param_t)
#define NC650_CPLD_SLOTADDR_GET		_IOR (NC650_CPLD_IOCTL_BASE, 2, unsigned)
#define NC650_CPLD_HDLC_MODE_GET	_IOR (NC650_CPLD_IOCTL_BASE, 3, unsigned)
#define NC650_CPLD_HDLC_MODE_SET	_IOW (NC650_CPLD_IOCTL_BASE, 4, unsigned)

#ifdef __KERNEL__

typedef struct
{
  char scc3_rs232_mode; /* Select SCC3 RS232 mode		*/
  char scc3_rs485_mode; /* Select SCC3 RS485-Modus 		*/
  char smc1_rs232_mode; /* Select SMC1 RS232 mode 		*/
  char smc1_rs485_mode; /* Select SMC1 RS485 mode 		*/
  char scc4_rs485_mode; /* Select SCC4 RS485 mode 		*/
  char scc4_open_collector_mode;  /* Select SCC4 busmode 	*/
  char i2c_bus1;        /* Select SCLC for bus 1 		*/
  char i2c_bus2;        /* Select SCLC for bus 2 		*/
  char reset_bus_on;    /* Set reset bus signal 		*/
  char reset_bus_off;   /* Clear reset bus signal 		*/
  char wdt_on;          /* Set WDT signal 			*/
  char wdt_off;         /* Clear WDT signal 			*/
  char err_led_on;      /* Set error led 			*/
  char err_led_off;     /* Clear error led 			*/
  char status_led;      /* write-only status led latch 		*/
  char status_reg_read; /* read-only status register (see below)*/
  char sync_bus_on;     /* Set bus sync signal 			*/
  char sync_bus_off;    /* Clear bus sync signal 		*/
  char sync_irq_off;    /* release sync-irq on bus 		*/
  char sync_irq_on;     /* lock sync-irq on bus 		*/
} cpld_t;

/* Bits of CPLD status register */
#define CPLD_SR_SCC3_RS485        0x80
#define CPLD_SR_SMC1_RS485        0x40
#define CPLD_SR_SCC4_OC           0x20
#define CPLD_SR_RESET_BUS         0x10
#define CPLD_SR_POWER_GOOD        0x08
#define CPLD_SR_SPS               0x04
#define CPLD_SR_PROG              0x02
#define CPLD_SR_PFAIL             0x01

/* status_led register definitions */
#define CPLD_SL_LED_RESERVED      0x01
#define CPLD_SL_RUN_LED           0x02
#define CPLD_SL_SCC3_LED_LINK     0x04
#define CPLD_SL_SCC3_LED_ACT      0x08
#define CPLD_SL_SMC1_LED_LINK     0x10
#define CPLD_SL_SMC1_LED_ACT      0x20
#define CPLD_SL_MII_LED_LINK      0x40
#define CPLD_SL_MII_LED_ACT       0x80
/* Error led is separate but we virtualize it */
#define CPLD_SL_ERR_LED          0x100

typedef struct
{
  char slot_addr_inv;
} peri_t;

extern unsigned char cpld_read_status(void);
extern void cpld_set_statusled(unsigned val);
extern void cpld_clear_statusled(unsigned val);
extern void cpld_toggle_statusled(unsigned val);
extern volatile cpld_t *cpld_get_ptr(void);

#endif /* __KERNEL__ */

#endif	/* _nc650_cpld_h */
