/***********************************************************************
 *
 * (C) Copyright 2004
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 ***********************************************************************/
/*
 * The purpose of this header file is to provide an interface for the
 * driver of the watchdog timer wdt. In essence this interface
 * consists of the three macros WDT_INIT, WDT_SERVICE,
 * WDT_CLOSE, and its functionality is described as follows:
 *
 * WDT_INIT:     opens the driver and initializes the timer to
 *                      300 seconds;
 *
 * WDT_SERVICE:  writes the value defined by the macro
 *                      WDT_DEF_SERVICE_TIME to the variable,
 *                      which serves as a timer counter;
 *
 * WDT_CLOSE:    closes the watchdog driver;
 *
 * Finally there is a macro called WDT_SET_SERVICE_TIME(sec)
 * for altering the value written to the timer counter to a value,
 * which is specified by sec.
 */


#ifndef _wdt_h
#define _wdt_h

typedef struct	wdt_param {
	unsigned chainid;
	unsigned long timer_count[3];
	int action[3];
	int signal;
} wdt_param_t;

/* Constants for the action[] fields */
#define WDT_ACTION_SIGNAL  1
#define WDT_ACTION_KILL    2
#define WDT_ACTION_REBOOT  3
#define WDT_ACTION_RESET   4

#define	WDT_IOCTL_BASE	'W'

#define WDT_OPEN_ONLY	_IO (WDT_IOCTL_BASE, 0)
#define WDT_ALWAYS	_IO (WDT_IOCTL_BASE, 1)
#define WDT_REGISTER	_IOW(WDT_IOCTL_BASE, 2, wdt_param_t)
#define WDT_RESET	_IOW(WDT_IOCTL_BASE, 3, int)
#define WDT_UNREGISTER	_IOW(WDT_IOCTL_BASE, 4, int)

#ifndef	__KERNEL__

#include <fcntl.h>
#include <unistd.h>
#include <linux/ioctl.h>

#define WDT_DEVICE "/dev/watchdog"
#define WDT_DEF_SERVICE_TIME 300

int wdt_fd;
int wdt_value = WDT_DEF_SERVICE_TIME;

#define WDT_INIT (wdt_fd = open(WDT_DEVICE, O_RDWR, 0))

#define WDT_SET_SERVICE_TIME(sec) wdt_value = (sec);

#define WDT_SERVICE write(wdt_fd, (char *) &wdt_value, sizeof (wdt_value))

#define WDT_CLOSE close(wdt_fd)

#endif	/* __KERNEL__ */

#endif	/* _wdt_h */
